/**
 * 
 */
package com.wieseke.cptk.common.util;

import com.wieseke.cptk.common.api.INode;

/**
 * @author Nicolas Wieseke
 * 
 */
public class NodeUtils {

	/**
	 * @param childCandidate
	 * @param parentCandidate
	 * @return true if the childCandidate is a child of the parentCandidate
	 */
	public static boolean isChild(INode childCandidate, INode parentCandidate) {
		return childCandidate.getNumber() > parentCandidate.getNumber()
				&& childCandidate.getNumber() <= parentCandidate.getNumber()
						+ parentCandidate.getChildrenCount();
	}

	/**
	 * @param node1
	 * @param node2
	 * @return the common ancestor of both nodes
	 */
	public static INode getCommonAncestor(INode node1, INode node2) {
		if (node1 == null || node2 == null)
			return null;
		if (node1.equals(node2))
			return node1;
		else if (node2.getParent() == null || isChild(node1, node2))
			return node2;
		else if (node1.getParent() == null || isChild(node2, node1))
			return node1;
		else
			return getCommonAncestor(node1.getParent(), node2.getParent());
	}

	/**
	 * @param node1
	 * @param node2
	 * @return the path length between node1 and node2
	 */
	public static int getPathLength(INode node1, INode node2) {
		INode lca = getCommonAncestor(node1, node2);
		if (lca == null) {
			return -1;
		}
		int counter = 0;
		INode parent = node1;
		while (parent != null && !parent.equals(lca)) {
			counter++;
			parent = parent.getParent();
		}
		parent = node2;
		while (parent != null && !parent.equals(lca)) {
			counter++;
			parent = parent.getParent();
		}
		return counter;
	}

	/**
	 * @param node
	 * @return the root node of the tree containing the node
	 */
	public static INode getRootNode(INode node) {
		while (node.getParent()!=null){
			node = node.getParent();
		}
		return node;
	}

	public static String getEdgeList(INode node) {
		String edgeList = "";
		for (INode child : node.getChildren()) {
			edgeList += child.getNumber() + " " + node.getNumber() + "\n";
			edgeList += getEdgeList(child);
		}
		return edgeList;
	}
}
