/**
 * 
 */
package com.wieseke.cptk.input.action;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.input.constants.InputActionConstants;
import com.wieseke.cptk.input.operation.EditChangeHostRankOperation;
import com.wieseke.cptk.input.operation.EditChangeParasiteRankOperation;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputHostNodeControl;
import com.wieseke.cptk.input.viewer.InputNodeControl;
import com.wieseke.cptk.input.viewer.InputParasiteNodeControl;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditChangeRankAction extends InputAction {

	/**
	 * default constructor
	 */
	public EditChangeRankAction() {
		super(InputActionConstants.CHANGE_RANK_TEXT);
	}

	/**
	 * constructor
	 * 
	 * @param viewer
	 * @param node
	 */
	public EditChangeRankAction(InputCophylogenyViewer viewer,
			InputNodeControl node) {
		super(InputActionConstants.CHANGE_RANK_TEXT);
		this.viewer = viewer;
		this.node = node;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	
	public void run() {
		if (this.node != null) {
			IUndoableOperation operation = null;
			if (this.node instanceof InputHostNodeControl) {
				operation = new EditChangeHostRankOperation(getViewer(),
						((InputHostNodeControl) this.node).getNode());
			} else if (this.node instanceof InputParasiteNodeControl) {
				operation = new EditChangeParasiteRankOperation(getViewer(),
						((InputParasiteNodeControl) this.node).getNode());
			}
			operation.addContext(this.viewer.getUndoContext());
			try {
				PlatformUI.getWorkbench().getOperationSupport()
						.getOperationHistory().execute(operation, null, null);
			} catch (ExecutionException e) {
				MessageDialog.openError(this.viewer.getShell(),
						"Change Rank Error",
						"Exception while changing the rank of node "
								+ this.node.getNode().getLabel() + ": "
								+ e.getMessage());
			}
		}
	}
}