/**
 * 
 */
package com.wieseke.cptk.input.action;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.input.constants.InputActionConstants;
import com.wieseke.cptk.input.operation.EditCutHostSubtreeOperation;
import com.wieseke.cptk.input.operation.EditCutParasiteSubtreeOperation;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputHostNodeControl;
import com.wieseke.cptk.input.viewer.InputNodeControl;
import com.wieseke.cptk.input.viewer.InputParasiteNodeControl;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditCutSubtreeAction extends InputAction {

	/**
	 * default constructor
	 */
	public EditCutSubtreeAction() {
		super(InputActionConstants.CUT_SUBTREE_TEXT);
	}

	/**
	 * constructor
	 * 
	 * @param viewer
	 * @param node
	 */
	public EditCutSubtreeAction(InputCophylogenyViewer viewer,
			InputNodeControl node) {
		super(InputActionConstants.CUT_SUBTREE_TEXT);
		this.viewer = viewer;
		this.node = node;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	
	public void run() {
		if (this.node != null) {
			IUndoableOperation operation = null;
			if (this.node instanceof InputHostNodeControl) {
				operation = new EditCutHostSubtreeOperation(getViewer(),
						((InputHostNodeControl) this.node).getNode());
			} else if (this.node instanceof InputParasiteNodeControl) {
				operation = new EditCutParasiteSubtreeOperation(getViewer(),
						((InputParasiteNodeControl) this.node).getNode());
			}
			operation.addContext(this.viewer.getUndoContext());
			try {
				PlatformUI.getWorkbench().getOperationSupport()
						.getOperationHistory().execute(operation, null, null);
			} catch (ExecutionException e) {
				MessageDialog.openError(this.viewer.getShell(),
						"Delete Node Error",
						"Exception while cutting the subtree at node "
								+ this.node.getNode().getLabel() + ": "
								+ e.getMessage());
			}
		}
	}

	
	public void setEnabled(boolean enabled) {
		if (enabled && node != null && node.getNode().getParent() != null) {
			super.setEnabled(true);
		} else {
			super.setEnabled(false);
		}
	}
}