/**
 * 
 */
package com.wieseke.cptk.input.action;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.input.constants.InputActionConstants;
import com.wieseke.cptk.input.operation.EditDeleteHostAssociationsOperation;
import com.wieseke.cptk.input.operation.EditDeleteParasiteAssociationsOperation;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputHostNodeControl;
import com.wieseke.cptk.input.viewer.InputNodeControl;
import com.wieseke.cptk.input.viewer.InputParasiteNodeControl;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditDeleteAssociationsAction extends InputAction {

	/**
	 * default constructor
	 */
	public EditDeleteAssociationsAction() {
		super(InputActionConstants.DELETE_ASSOCIATIONS_TEXT);
	}

	/**
	 * constructor
	 * 
	 * @param viewer
	 * @param node
	 */
	public EditDeleteAssociationsAction(InputCophylogenyViewer viewer,
			InputNodeControl node) {
		super(InputActionConstants.DELETE_ASSOCIATIONS_TEXT);
		this.viewer = viewer;
		this.node = node;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	
	public void run() {
		if (this.node != null) {
			IUndoableOperation operation = null;
			if (this.node instanceof InputHostNodeControl) {
				operation = new EditDeleteHostAssociationsOperation(
						getViewer(), ((InputHostNodeControl) this.node)
								.getNode());
			} else if (this.node instanceof InputParasiteNodeControl) {
				operation = new EditDeleteParasiteAssociationsOperation(
						getViewer(), ((InputParasiteNodeControl) this.node)
								.getNode());
			}
			operation.addContext(this.viewer.getUndoContext());
			try {
				PlatformUI.getWorkbench().getOperationSupport()
						.getOperationHistory().execute(operation, null, null);
			} catch (ExecutionException e) {
				MessageDialog.openError(this.viewer.getShell(),
						"Delete Associations Error",
						"Exception while deleting the associations of node "
								+ this.node.getNode().getLabel() + ": "
								+ e.getMessage());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#setEnabled(boolean)
	 */
	
	public void setEnabled(boolean enabled) {
		if (enabled && node != null) {
			super.setEnabled(true);
		} else {
			super.setEnabled(false);
		}
	}

}