/**
 * 
 */
package com.wieseke.cptk.input.action;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.input.constants.InputActionConstants;
import com.wieseke.cptk.input.operation.EditRearrangeChildPositionOperation;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;
import com.wieseke.cptk.input.viewer.InputNodeControl;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditRearrangeChildPositionAction extends InputAction {

	/**
	 * default constructor
	 */
	public EditRearrangeChildPositionAction() {
		super(InputActionConstants.REARRANGE_CHILD_POSITION_TEXT);
	}

	/**
	 * constructor
	 * 
	 * @param viewer
	 * @param node
	 */
	public EditRearrangeChildPositionAction(InputCophylogenyViewer viewer,
			InputNodeControl node) {
		super(InputActionConstants.REARRANGE_CHILD_POSITION_TEXT);
		this.viewer = viewer;
		this.node = node;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#run()
	 */
	
	public void run() {
		if (this.node != null) {
			EditRearrangeChildPositionOperation operation = new EditRearrangeChildPositionOperation(
					getViewer(), this.node.getNode());
			operation.addContext(this.viewer.getUndoContext());
			try {
				PlatformUI.getWorkbench().getOperationSupport()
						.getOperationHistory().execute(operation, null, null);
			} catch (ExecutionException e) {
				MessageDialog.openError(this.viewer.getShell(),
						"Rearrange Child Order Error",
						"Exception while rearranging the order of the node's children "
								+ this.node.getNode().getLabel() + ": "
								+ e.getMessage());
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.jface.action.Action#setEnabled(boolean)
	 */
	
	public void setEnabled(boolean enabled) {
		if (enabled && node != null && !node.getNode().isLeaf()) {
			super.setEnabled(true);
		} else {
			super.setEnabled(false);
		}
	}
}