/**
 * 
 */
package com.wieseke.cptk.input.action;

import org.eclipse.jface.resource.ImageDescriptor;

import com.wieseke.cptk.app.CPTKApplication;
import com.wieseke.cptk.input.constants.InputActionConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public abstract class InputCophylogenyViewerActionFactory {

	// node actions
	public static final InputCophylogenyViewerActionFactory SELECT_FOLD_NODE = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.FOLD_NODE_TEXT;
			InputAction action = new SelectEditModeFoldNodeAction(text);
			action.setId(InputActionConstants.FOLD_NODE_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.FOLD_NODE_IMAGEPATH));
			action.setToolTipText(InputActionConstants.FOLD_NODE_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_ADD_NODE = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.ADD_NODE_TEXT;
			InputAction action = new SelectEditModeAddNodeAction(text);
			action.setId(InputActionConstants.ADD_NODE_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.ADD_NODE_IMAGEPATH));
			action.setToolTipText(InputActionConstants.ADD_NODE_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_DELETE_NODE = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.DELETE_NODE_TEXT;
			InputAction action = new SelectEditModeDeleteNodeAction(text);
			action.setId(InputActionConstants.DELETE_NODE_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.DELETE_NODE_IMAGEPATH));
			action.setToolTipText(InputActionConstants.DELETE_NODE_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_CUT_SUBTREE = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.CUT_SUBTREE_TEXT;
			InputAction action = new SelectEditModeCutSubtreeAction(text);
			action.setId(InputActionConstants.CUT_SUBTREE_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.CUT_SUBTREE_IMAGEPATH));
			action.setToolTipText(InputActionConstants.CUT_SUBTREE_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_CHANGE_LABEL = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.CHANGE_LABEL_TEXT;
			InputAction action = new SelectEditModeChangeLabelAction(text);
			action.setId(InputActionConstants.CHANGE_LABEL_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.CHANGE_LABEL_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.CHANGE_LABEL_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_CHANGE_RANK = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.CHANGE_RANK_TEXT;
			InputAction action = new SelectEditModeChangeRankAction(text);
			action.setId(InputActionConstants.CHANGE_RANK_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.CHANGE_RANK_IMAGEPATH));
			action.setToolTipText(InputActionConstants.CHANGE_RANK_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_ADD_ASSOCIATION = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.ADD_ASSOCIATION_TEXT;
			InputAction action = new SelectEditModeAddLinksAction(text);
			action.setId(InputActionConstants.ADD_ASSOCIATION_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.ADD_ASSOCIATION_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.ADD_ASSOCIATION_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_DELETE_ASSOCIATIONS = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.DELETE_ASSOCIATIONS_TEXT;
			InputAction action = new SelectEditModeDeleteLinksAction(text);
			action.setId(InputActionConstants.DELETE_ASSOCIATIONS_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.DELETE_ASSOCIATIONS_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.DELETE_ASSOCIATIONS_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_REARRANGE_CHILD_POSITION = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.REARRANGE_CHILD_POSITION_TEXT;
			InputAction action = new SelectEditModeRearrangeChildPositionAction(
					text);
			action.setId(InputActionConstants.REARRANGE_CHILD_POSITION_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.REARRANGE_CHILD_POSITION_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.REARRANGE_CHILD_POSITION_TOOLTIPTEXT);
			return action;
		}
	};

	// tree actions
	public static final InputCophylogenyViewerActionFactory SELECT_BOTH = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.SELECT_BOTH_TEXT;
			InputAction action = new SelectTreeModeBothAction(text);
			action.setId(InputActionConstants.SELECT_BOTH_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.SELECT_BOTH_IMAGEPATH));
			action.setToolTipText(InputActionConstants.SELECT_BOTH_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_HOST = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.SELECT_HOST_TEXT;
			InputAction action = new SelectTreeModeHostAction(text);
			action.setId(InputActionConstants.SELECT_HOST_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.SELECT_HOST_IMAGEPATH));
			action.setToolTipText(InputActionConstants.SELECT_HOST_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory SELECT_PARASITE = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.SELECT_PARASITE_TEXT;
			InputAction action = new SelectTreeModeParasiteAction(text);
			action.setId(InputActionConstants.SELECT_PARASITE_ID);
			action.setImageDescriptor(ImageDescriptor.createFromFile(
					CPTKApplication.class,
					InputActionConstants.SELECT_PARASITE_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.SELECT_PARASITE_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory CHANGE_COPHYLOGENY_PROPERTIES = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.CHANGE_COPHYLOGENY_PROPERTIES_TEXT;
			InputAction action = new ChangeCophylogenyPropertiesAction(text);
			action.setId(InputActionConstants.CHANGE_COPHYLOGENY_PROPERTIES_ID);
			action
					.setImageDescriptor(ImageDescriptor
							.createFromFile(
									CPTKApplication.class,
									InputActionConstants.CHANGE_COPHYLOGENY_PROPERTIES_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.CHANGE_COPHYLOGENY_PROPERTIES_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory RUN_COPHYLOGENY_RECONSTRUCTION = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.RUN_COPHYLOGENY_RECONSTRUCTION_TEXT;
			InputAction action = new RunCophylogenyReconstructionAction(text);
			action
					.setId(InputActionConstants.RUN_COPHYLOGENY_RECONSTRUCTION_ID);
			action
					.setImageDescriptor(ImageDescriptor
							.createFromFile(
									CPTKApplication.class,
									InputActionConstants.RUN_COPHYLOGENY_RECONSTRUCTION_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.RUN_COPHYLOGENY_RECONSTRUCTION_TOOLTIPTEXT);
			return action;
		}
	};

	public static final InputCophylogenyViewerActionFactory RUN_RANDOM_COPHYLOGENY_RECONSTRUCTION = new InputCophylogenyViewerActionFactory() {
		public InputAction create() {

			String text = InputActionConstants.RUN_RANDOM_COPHYLOGENY_RECONSTRUCTION_TEXT;
			InputAction action = new RunRandomCophylogenyReconstructionAction(
					text);
			action
					.setId(InputActionConstants.RUN_RANDOM_COPHYLOGENY_RECONSTRUCTION_ID);
			action
					.setImageDescriptor(ImageDescriptor
							.createFromFile(
									CPTKApplication.class,
									InputActionConstants.RUN_RANDOM_COPHYLOGENY_RECONSTRUCTION_IMAGEPATH));
			action
					.setToolTipText(InputActionConstants.RUN_RANDOM_COPHYLOGENY_RECONSTRUCTION_TOOLTIPTEXT);
			return action;
		}
	};

	public abstract InputAction create();
}
