package com.wieseke.cptk.input.editor;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.action.ActionContributionItem;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.operations.RedoActionHandler;
import org.eclipse.ui.operations.UndoActionHandler;
import org.eclipse.ui.part.MultiPageEditorActionBarContributor;
import org.eclipse.ui.texteditor.ITextEditor;
import org.eclipse.ui.texteditor.ITextEditorActionConstants;

import com.wieseke.cptk.common.action.CommonAction;
import com.wieseke.cptk.common.action.CophylogenyViewerActionFactory;
import com.wieseke.cptk.common.action.PrintAction;
import com.wieseke.cptk.input.action.InputAction;
import com.wieseke.cptk.input.action.InputCophylogenyViewerActionFactory;
import com.wieseke.cptk.input.action.SelectEditModeSelectionAction;
import com.wieseke.cptk.input.action.SelectTreeModeSelectionAction;

/**
 * Manages the installation/deinstallation of global actions for multi-page
 * editors. Responsible for the redirection of global actions to the active
 * editor. Multi-page contributor replaces the contributors for the individual
 * editors in the multi-page editor.
 */
public class MultiPageInputEditorActionBarContributor extends
		MultiPageEditorActionBarContributor {
	private IEditorPart activeEditorPart;

	private List<CommonAction> exportActions = new ArrayList<CommonAction>();
	private List<CommonAction> zoomActions = new ArrayList<CommonAction>();
	SelectEditModeSelectionAction selectEditModeSelectionAction;
	private List<InputAction> selectEditModeActions = new ArrayList<InputAction>();
	private List<CommonAction> nodeActions = new ArrayList<CommonAction>();
	SelectTreeModeSelectionAction selectTreeModeSelectionAction;
	private List<InputAction> selectTreeModeActions = new ArrayList<InputAction>();
	private List<CommonAction> treeActions = new ArrayList<CommonAction>();
	private List<InputAction> cophylogenyActions = new ArrayList<InputAction>();
	private PrintAction printAction = new PrintAction("Print...");

	private UndoActionHandler undoAction;
	private RedoActionHandler redoAction;

	/**
	 * Creates a multi-page contributor.
	 */
	public MultiPageInputEditorActionBarContributor() {
		super();
		createActions();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.ui.part.EditorActionBarContributor#init(org.eclipse.ui.
	 * IActionBars, org.eclipse.ui.IWorkbenchPage)
	 */

	public void init(IActionBars bars, IWorkbenchPage page) {
		super.init(bars, page);
		initActions();

	}

	/**
	 * Returns the action registed with the given text editor.
	 * 
	 * @return IAction or null if editor is null.
	 */
	protected IAction getAction(ITextEditor editor, String actionID) {
		return (editor == null ? null : editor.getAction(actionID));
	}

	/*
	 * (non-JavaDoc) Method declared in
	 * AbstractMultiPageEditorActionBarContributor.
	 */

	public void setActivePage(IEditorPart part) {
		if (activeEditorPart == part)
			return;

		activeEditorPart = part;

		IActionBars actionBars = getActionBars();
		if (actionBars != null) {

			ITextEditor editor = (part instanceof ITextEditor) ? (ITextEditor) part
					: null;

			actionBars.setGlobalActionHandler(ActionFactory.DELETE.getId(),
					getAction(editor, ITextEditorActionConstants.DELETE));
			actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(),
					getAction(editor, ITextEditorActionConstants.UNDO));
			actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(),
					getAction(editor, ITextEditorActionConstants.REDO));
			actionBars.setGlobalActionHandler(ActionFactory.CUT.getId(),
					getAction(editor, ITextEditorActionConstants.CUT));
			actionBars.setGlobalActionHandler(ActionFactory.COPY.getId(),
					getAction(editor, ITextEditorActionConstants.COPY));
			actionBars.setGlobalActionHandler(ActionFactory.PASTE.getId(),
					getAction(editor, ITextEditorActionConstants.PASTE));
			actionBars.setGlobalActionHandler(ActionFactory.SELECT_ALL.getId(),
					getAction(editor, ITextEditorActionConstants.SELECT_ALL));
			actionBars.setGlobalActionHandler(ActionFactory.FIND.getId(),
					getAction(editor, ITextEditorActionConstants.FIND));
			actionBars.setGlobalActionHandler(ActionFactory.PRINT.getId(),
					getAction(editor, ITextEditorActionConstants.PRINT));

			// actionBars.updateActionBars();

			if (!(part instanceof VisualInputEditor)) {
				disableInputActions();
			} else {
				enableInputActions();
				initUndoRedoActions((VisualInputEditor) part);
				enableUndoRedoActions(actionBars);
				actionBars.setGlobalActionHandler(ActionFactory.PRINT.getId(),
						printAction);
			}

			actionBars.updateActionBars();
		}
	}

	/**
	 * create the cophylogeny action
	 */
	private void createActions() {
		CophylogenyViewerActionFactory exportActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.PRINT,
				CophylogenyViewerActionFactory.EXPORT_SVG };
		CophylogenyViewerActionFactory zoomActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.ZOOM_RESET,
				CophylogenyViewerActionFactory.ZOOM_IN,
				CophylogenyViewerActionFactory.ZOOM_OUT };
		InputCophylogenyViewerActionFactory selectEditModeActionsToCreate[] = new InputCophylogenyViewerActionFactory[] {
				InputCophylogenyViewerActionFactory.SELECT_FOLD_NODE,
				InputCophylogenyViewerActionFactory.SELECT_ADD_NODE,
				InputCophylogenyViewerActionFactory.SELECT_DELETE_NODE,
				InputCophylogenyViewerActionFactory.SELECT_CUT_SUBTREE,
				InputCophylogenyViewerActionFactory.SELECT_CHANGE_LABEL,
				InputCophylogenyViewerActionFactory.SELECT_CHANGE_RANK,
				InputCophylogenyViewerActionFactory.SELECT_ADD_ASSOCIATION,
				InputCophylogenyViewerActionFactory.SELECT_DELETE_ASSOCIATIONS,
				InputCophylogenyViewerActionFactory.SELECT_REARRANGE_CHILD_POSITION };
		CophylogenyViewerActionFactory nodeActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.ADJUST_CHILD_NODES,
				CophylogenyViewerActionFactory.SHOW_HOST_LABELS,
				CophylogenyViewerActionFactory.SHOW_PARASITE_LABELS,
				CophylogenyViewerActionFactory.SHOW_RANK_LABELS,
				CophylogenyViewerActionFactory.RECALCULATE_NODE_POSITIONS,
				CophylogenyViewerActionFactory.RECALCULATE_NODE_ORDER };
		InputCophylogenyViewerActionFactory selectTreeModeActionsToCreate[] = new InputCophylogenyViewerActionFactory[] {
				InputCophylogenyViewerActionFactory.SELECT_BOTH,
				InputCophylogenyViewerActionFactory.SELECT_HOST,
				InputCophylogenyViewerActionFactory.SELECT_PARASITE };
		CophylogenyViewerActionFactory treeActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.FLIP_HORIZONTAL,
				CophylogenyViewerActionFactory.FLIP_VERTICAL,
				CophylogenyViewerActionFactory.ROTATE_LEFT,
				CophylogenyViewerActionFactory.ROTATE_RIGHT };
		InputCophylogenyViewerActionFactory cophylogenyActionsToCreate[] = new InputCophylogenyViewerActionFactory[] {
				InputCophylogenyViewerActionFactory.CHANGE_COPHYLOGENY_PROPERTIES,
				InputCophylogenyViewerActionFactory.RUN_COPHYLOGENY_RECONSTRUCTION,
				InputCophylogenyViewerActionFactory.RUN_RANDOM_COPHYLOGENY_RECONSTRUCTION };

		for (CophylogenyViewerActionFactory factory : exportActionsToCreate) {
			CommonAction action = factory.create();
			this.exportActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : zoomActionsToCreate) {
			CommonAction action = factory.create();
			this.zoomActions.add(action);
		}
		for (InputCophylogenyViewerActionFactory factory : selectEditModeActionsToCreate) {
			InputAction action = factory.create();
			this.selectEditModeActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : nodeActionsToCreate) {
			CommonAction action = factory.create();
			this.nodeActions.add(action);
		}
		for (InputCophylogenyViewerActionFactory factory : selectTreeModeActionsToCreate) {
			InputAction action = factory.create();
			this.selectTreeModeActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : treeActionsToCreate) {
			CommonAction action = factory.create();
			this.treeActions.add(action);
		}
		for (InputCophylogenyViewerActionFactory factory : cophylogenyActionsToCreate) {
			InputAction action = factory.create();
			this.cophylogenyActions.add(action);
		}

	}

	/**
	 * registers the listener
	 */
	private void initActions() {
		for (CommonAction action : this.exportActions) {
			getPage().addSelectionListener(action);
		}
		for (CommonAction action : this.zoomActions) {
			getPage().addSelectionListener(action);
		}
		for (InputAction action : this.selectEditModeActions) {
			getPage().addSelectionListener(action);
		}
		for (CommonAction action : this.nodeActions) {
			getPage().addSelectionListener(action);
		}
		for (InputAction action : this.selectTreeModeActions) {
			getPage().addSelectionListener(action);
		}
		for (CommonAction action : this.treeActions) {
			getPage().addSelectionListener(action);
		}
		for (InputAction action : this.cophylogenyActions) {
			getPage().addSelectionListener(action);
		}
		getPage().addSelectionListener(this.selectEditModeSelectionAction);
		getPage().addSelectionListener(this.selectTreeModeSelectionAction);
		getPage().addSelectionListener(this.printAction);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.EditorActionBarContributor#contributeToMenu(org.eclipse
	 * .jface.action.IMenuManager)
	 */

	public void contributeToMenu(IMenuManager manager) {
		IMenuManager menu = new MenuManager("&Cophylogeny");
		manager.prependToGroup(IWorkbenchActionConstants.MB_ADDITIONS, menu);
		for (CommonAction action : this.exportActions) {
			menu.add(action);
		}
		menu.add(new Separator());
		for (CommonAction action : this.zoomActions) {
			menu.add(action);
		}
		menu.add(new Separator());
		IMenuManager selectEditModeActionMenu = new MenuManager(
				"Select &edit mode");
		for (InputAction action : this.selectEditModeActions) {
			selectEditModeActionMenu.add(action);
		}
		menu.add(selectEditModeActionMenu);
		menu.add(new Separator());
		for (CommonAction action : this.nodeActions) {
			menu.add(action);
		}
		menu.add(new Separator());
		IMenuManager selectTreeModeActionMenu = new MenuManager(
				"Select &tree mode");
		for (InputAction action : this.selectTreeModeActions) {
			selectTreeModeActionMenu.add(action);
		}
		menu.add(selectTreeModeActionMenu);
		menu.add(new Separator());
		for (CommonAction action : this.treeActions) {
			menu.add(action);
		}
		menu.add(new Separator());
		for (InputAction action : this.cophylogenyActions) {
			menu.add(action);
		}
		menu.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.EditorActionBarContributor#contributeToToolBar(org
	 * .eclipse.jface.action.IToolBarManager)
	 */

	public void contributeToToolBar(IToolBarManager manager) {
		manager.add(new Separator());
		for (CommonAction action : this.exportActions) {
			manager.add(action);
		}
		manager.add(new Separator());
		for (CommonAction action : this.zoomActions) {
			manager.add(action);
		}
		manager.add(new Separator());
		selectEditModeSelectionAction = new SelectEditModeSelectionAction(
				"Select edit mode");
		for (InputAction action : this.selectEditModeActions) {
			selectEditModeSelectionAction.addActionToMenu(action);
		}
		ActionContributionItem selectEditModeSelectionItem = new ActionContributionItem(
				selectEditModeSelectionAction);
		selectEditModeSelectionItem
				.setMode(ActionContributionItem.MODE_FORCE_TEXT);
		manager.add(selectEditModeSelectionItem);
		manager.add(new Separator());
		for (CommonAction action : this.nodeActions) {
			manager.add(action);
		}
		manager.add(new Separator());
		selectTreeModeSelectionAction = new SelectTreeModeSelectionAction(
				"Select tree mode");
		for (InputAction action : this.selectTreeModeActions) {
			selectTreeModeSelectionAction.addActionToMenu(action);
		}
		ActionContributionItem selectTreeModeSelectionItem = new ActionContributionItem(
				selectTreeModeSelectionAction);
		selectTreeModeSelectionItem
				.setMode(ActionContributionItem.MODE_FORCE_TEXT);
		manager.add(selectTreeModeSelectionItem);
		manager.add(new Separator());
		for (CommonAction action : this.treeActions) {
			manager.add(action);
		}
		manager.add(new Separator());
		for (InputAction action : this.cophylogenyActions) {
			manager.add(action);
		}
		manager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));

	}

	private void initUndoRedoActions(VisualInputEditor visualEditor) {
		if (visualEditor.getInputCophylogenyViewer() != null) {
			this.undoAction = new UndoActionHandler(visualEditor.getSite(),
					visualEditor.getInputCophylogenyViewer().getUndoContext());
			this.redoAction = new RedoActionHandler(visualEditor.getSite(),
					visualEditor.getInputCophylogenyViewer().getUndoContext());
		}
	}

	private void enableInputActions() {
		for (CommonAction action : this.exportActions) {
			action.setEnabled(true);
		}
		for (CommonAction action : this.zoomActions) {
			action.setEnabled(true);
		}
		for (InputAction action : this.selectEditModeActions) {
			action.setEnabled(true);
		}
		for (CommonAction action : this.nodeActions) {
			action.setEnabled(true);
		}
		for (InputAction action : this.selectTreeModeActions) {
			action.setEnabled(true);
		}
		for (CommonAction action : this.treeActions) {
			action.setEnabled(true);
		}
		for (InputAction action : this.cophylogenyActions) {
			action.setEnabled(true);
		}
		selectEditModeSelectionAction.setEnabled(true);
		selectTreeModeSelectionAction.setEnabled(true);
		printAction.setEnabled(true);
	}

	private void enableUndoRedoActions(IActionBars actionBars) {
		actionBars.setGlobalActionHandler(ActionFactory.UNDO.getId(),
				undoAction);
		actionBars.setGlobalActionHandler(ActionFactory.REDO.getId(),
				redoAction);
	}

	private void disableInputActions() {
		for (CommonAction action : this.exportActions) {
			action.setEnabled(false);
		}
		for (CommonAction action : this.zoomActions) {
			action.setEnabled(false);
		}
		for (InputAction action : this.selectEditModeActions) {
			action.setEnabled(false);
		}
		for (CommonAction action : this.nodeActions) {
			action.setEnabled(false);
		}
		for (InputAction action : this.selectTreeModeActions) {
			action.setEnabled(false);
		}
		for (CommonAction action : this.treeActions) {
			action.setEnabled(false);
		}
		for (InputAction action : this.cophylogenyActions) {
			action.setEnabled(false);
		}
		selectEditModeSelectionAction.setEnabled(false);
		selectTreeModeSelectionAction.setEnabled(false);
		printAction.setEnabled(false);
	}
}
