/**
 * 
 */
package com.wieseke.cptk.input.editor;

import java.io.File;
import java.io.InputStream;
import java.net.URI;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.action.IMenuListener;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.action.MenuManager;
import org.eclipse.jface.action.Separator;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.ISelectionService;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.ide.FileStoreEditorInput;
import org.eclipse.ui.part.EditorPart;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.texteditor.InfoForm;

import com.wieseke.cptk.common.action.CommonAction;
import com.wieseke.cptk.common.action.CophylogenyViewerActionFactory;
import com.wieseke.cptk.input.action.InputAction;
import com.wieseke.cptk.input.action.InputCophylogenyViewerActionFactory;
import com.wieseke.cptk.input.action.InputNodeControlActionFactory;
import com.wieseke.cptk.input.constants.InputViewerConstants;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.format.nexus.NexusDeserializer;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;

/**
 * @author Nicolas Wieseke
 * 
 */
@SuppressWarnings("deprecation")
public class VisualInputEditor extends EditorPart {

	public static final String ID = "com.wieseke.cptk.input.editor.VisualInputEditor";

	private InputCophylogenyViewer inputCophylogenyViewer;
	private boolean dirty;

	private List<CommonAction> exportActions = new ArrayList<CommonAction>();
	private List<InputAction> nodeActions = new ArrayList<InputAction>();
	private List<CommonAction> zoomActions = new ArrayList<CommonAction>();
	private List<InputAction> selectEditModeActions = new ArrayList<InputAction>();
	private List<CommonAction> nodesActions = new ArrayList<CommonAction>();
	private List<InputAction> selectTreeModeActions = new ArrayList<InputAction>();
	private List<CommonAction> treeActions = new ArrayList<CommonAction>();
	private List<InputAction> cophylogenyActions = new ArrayList<InputAction>();

	/**
	 * default constructor
	 */
	public VisualInputEditor() {
		this.inputCophylogenyViewer = null;
		this.dirty = false;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.ui.part.EditorPart#doSave(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */

	public void doSave(IProgressMonitor monitor) {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#doSaveAs()
	 */

	public void doSaveAs() {
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#init(org.eclipse.ui.IEditorSite,
	 * org.eclipse.ui.IEditorInput)
	 */

	public void init(IEditorSite site, IEditorInput input)
			throws PartInitException {
		setSite(site);
		setInput(input);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isDirty()
	 */

	public boolean isDirty() {
		return dirty;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.EditorPart#isSaveAsAllowed()
	 */

	public boolean isSaveAsAllowed() {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */

	public void createPartControl(Composite parent) {

		try {
			InputStream inputStream = null;
			String filename = getEditorInput().getToolTipText();
			if (getEditorInput() instanceof FileStoreEditorInput) {
				URI uri = ((FileStoreEditorInput) getEditorInput()).getURI();
				inputStream = FileUtils.openInputStream(new File(uri));
			} else if (getEditorInput() instanceof FileEditorInput) {
				filename = ((FileEditorInput) getEditorInput()).getPath()
						.toString();
				IFile file = org.eclipse.ui.ide.ResourceUtil
						.getFile(getEditorInput());
				inputStream = file.getContents();
			} else {
				IFile file = org.eclipse.ui.ide.ResourceUtil
						.getFile(getEditorInput());
				inputStream = file.getContents();
			}

			NexusDeserializer nexusDeserializer = new NexusDeserializer();

			InputCophylogeny cophylogeny = nexusDeserializer.deserialize(
					inputStream, filename);
			this.inputCophylogenyViewer = new InputCophylogenyViewer(parent,
					cophylogeny, getEditorInput().getToolTipText());
			getSite().setSelectionProvider(this.inputCophylogenyViewer);
			this.inputCophylogenyViewer
					.addPropertyChangeListener(new IPropertyChangeListener() {

						public void propertyChange(PropertyChangeEvent evtent) {
							if (evtent.getProperty().equals(
									InputViewerConstants.PROPERTY_DIRTY)
									&& !isDirty()) {
								if (dirty != (Boolean) evtent.getNewValue()) {
									dirty = (Boolean) evtent.getNewValue();
									firePropertyChange(IEditorPart.PROP_DIRTY);
								}
							}
						}

					});
			createActions();
			createNodeContextMenu();
			createViewerContextMenu();
			ISelectionService service = (ISelectionService) getSite()
					.getService(ISelectionService.class);
			service.addSelectionListener(new ISelectionListener() {

				public void selectionChanged(IWorkbenchPart part,
						ISelection selection) {
				}
			});
		} catch (Exception e) {
			InfoForm infoForm = new InfoForm(parent);
			infoForm.setInfo(e.getMessage());
		}
	}

	private void createActions() {
		CophylogenyViewerActionFactory exportActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.PRINT,
				CophylogenyViewerActionFactory.EXPORT_SVG };
		for (CophylogenyViewerActionFactory factory : exportActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.exportActions.add(action);
		}
		InputNodeControlActionFactory nodeActionsToCreate[] = new InputNodeControlActionFactory[] {
				InputNodeControlActionFactory.FOLD_NODE,
				InputNodeControlActionFactory.ADD_NODE,
				InputNodeControlActionFactory.DELETE_NODE,
				InputNodeControlActionFactory.CUT_SUBTREE,
				InputNodeControlActionFactory.CHANGE_LABEL,
				InputNodeControlActionFactory.CHANGE_RANK,
				InputNodeControlActionFactory.ADD_ASSOCIATION,
				InputNodeControlActionFactory.DELETE_ASSOCIATIONS,
				InputNodeControlActionFactory.REARRANGE_CHILD_POSITION };
		for (InputNodeControlActionFactory factory : nodeActionsToCreate) {
			InputAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.nodeActions.add(action);
		}
		CophylogenyViewerActionFactory zoomActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.ZOOM_RESET,
				CophylogenyViewerActionFactory.ZOOM_IN,
				CophylogenyViewerActionFactory.ZOOM_OUT };
		InputCophylogenyViewerActionFactory selectEditModeActionsToCreate[] = new InputCophylogenyViewerActionFactory[] {
				InputCophylogenyViewerActionFactory.SELECT_FOLD_NODE,
				InputCophylogenyViewerActionFactory.SELECT_ADD_NODE,
				InputCophylogenyViewerActionFactory.SELECT_DELETE_NODE,
				InputCophylogenyViewerActionFactory.SELECT_CUT_SUBTREE,
				InputCophylogenyViewerActionFactory.SELECT_CHANGE_LABEL,
				InputCophylogenyViewerActionFactory.SELECT_CHANGE_RANK,
				InputCophylogenyViewerActionFactory.SELECT_ADD_ASSOCIATION,
				InputCophylogenyViewerActionFactory.SELECT_DELETE_ASSOCIATIONS,
				InputCophylogenyViewerActionFactory.SELECT_REARRANGE_CHILD_POSITION };
		CophylogenyViewerActionFactory nodesActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.ADJUST_CHILD_NODES,
				CophylogenyViewerActionFactory.SHOW_HOST_LABELS,
				CophylogenyViewerActionFactory.SHOW_PARASITE_LABELS,
				CophylogenyViewerActionFactory.SHOW_RANK_LABELS,
				CophylogenyViewerActionFactory.RECALCULATE_NODE_POSITIONS,
				CophylogenyViewerActionFactory.RECALCULATE_NODE_ORDER };
		InputCophylogenyViewerActionFactory selectTreeModeActionsToCreate[] = new InputCophylogenyViewerActionFactory[] {
				InputCophylogenyViewerActionFactory.SELECT_BOTH,
				InputCophylogenyViewerActionFactory.SELECT_HOST,
				InputCophylogenyViewerActionFactory.SELECT_PARASITE };
		CophylogenyViewerActionFactory treeActionsToCreate[] = new CophylogenyViewerActionFactory[] {
				CophylogenyViewerActionFactory.FLIP_HORIZONTAL,
				CophylogenyViewerActionFactory.FLIP_VERTICAL,
				CophylogenyViewerActionFactory.ROTATE_LEFT,
				CophylogenyViewerActionFactory.ROTATE_RIGHT };
		InputCophylogenyViewerActionFactory cophylogenyActionsToCreate[] = new InputCophylogenyViewerActionFactory[] {
				InputCophylogenyViewerActionFactory.CHANGE_COPHYLOGENY_PROPERTIES,
				InputCophylogenyViewerActionFactory.RUN_COPHYLOGENY_RECONSTRUCTION,
				InputCophylogenyViewerActionFactory.RUN_RANDOM_COPHYLOGENY_RECONSTRUCTION };
		for (CophylogenyViewerActionFactory factory : zoomActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			// action.setViewer(this.inputCophylogenyViewer);
			this.zoomActions.add(action);
		}
		for (InputCophylogenyViewerActionFactory factory : selectEditModeActionsToCreate) {
			InputAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.selectEditModeActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : nodesActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.nodesActions.add(action);
		}
		for (InputCophylogenyViewerActionFactory factory : selectTreeModeActionsToCreate) {
			InputAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.selectTreeModeActions.add(action);
		}
		for (CophylogenyViewerActionFactory factory : treeActionsToCreate) {
			CommonAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.treeActions.add(action);
		}
		for (InputCophylogenyViewerActionFactory factory : cophylogenyActionsToCreate) {
			InputAction action = factory.create();
			getSite().getPage().addSelectionListener(action);
			this.cophylogenyActions.add(action);
		}
	}

	private void createNodeContextMenu() {
		MenuManager menuManager = new MenuManager("#NodePopupMenu");
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new IMenuListener() {

			public void menuAboutToShow(IMenuManager manager) {
				VisualInputEditor.this.fillNodeContextMenu(manager);
			}
		});
		Menu menu = menuManager.createContextMenu(this.inputCophylogenyViewer);
		this.inputCophylogenyViewer.setNodeContextMenu(menu);
		getSite().registerContextMenu(menuManager, this.inputCophylogenyViewer);
	}

	private void createViewerContextMenu() {
		MenuManager menuManager = new MenuManager("#ViewerPopupMenu");
		menuManager.setRemoveAllWhenShown(true);
		menuManager.addMenuListener(new IMenuListener() {

			public void menuAboutToShow(IMenuManager manager) {
				VisualInputEditor.this.fillViewerContextMenu(manager);
			}
		});
		Menu menu = menuManager.createContextMenu(this.inputCophylogenyViewer);
		this.inputCophylogenyViewer.setViewerMenu(menu);
		getSite().registerContextMenu(menuManager, this.inputCophylogenyViewer);
	}

	private void fillNodeContextMenu(IMenuManager menuManager) {
		boolean isEmpty = this.inputCophylogenyViewer.getSelection().isEmpty();
		for (InputAction action : this.nodeActions) {
			action.setEnabled(!isEmpty);
			menuManager.add(action);
		}
		menuManager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	private void fillViewerContextMenu(IMenuManager menuManager) {
		for (CommonAction action : this.exportActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		for (CommonAction action : this.zoomActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		IMenuManager selectEditModeActionMenu = new MenuManager(
				"Select &edit mode");
		for (InputAction action : this.selectEditModeActions) {
			action.setEnabled(true);
			selectEditModeActionMenu.add(action);
		}
		menuManager.add(selectEditModeActionMenu);
		menuManager.add(new Separator());
		for (CommonAction action : this.nodesActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		IMenuManager selectTreeModeActionMenu = new MenuManager(
				"Select &tree mode");
		for (InputAction action : this.selectTreeModeActions) {
			action.setEnabled(true);
			selectTreeModeActionMenu.add(action);
		}
		menuManager.add(selectTreeModeActionMenu);
		menuManager.add(new Separator());
		for (CommonAction action : this.treeActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator());
		for (InputAction action : this.cophylogenyActions) {
			action.setEnabled(true);
			menuManager.add(action);
		}
		menuManager.add(new Separator(IWorkbenchActionConstants.MB_ADDITIONS));
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */

	public void setFocus() {
		if (this.inputCophylogenyViewer != null) {
			this.inputCophylogenyViewer.setFocus();
		}
	}

	/**
	 * @return the inputCophylogeny
	 */
	public InputCophylogeny getInputCophylogeny() {
		return this.inputCophylogenyViewer.getCophylogeny();
	}

	/**
	 * @param inputCophylogeny
	 *            the inputCophylogeny to set
	 */
	public void setInputCophylogeny(InputCophylogeny inputCophylogeny) {
		this.inputCophylogenyViewer.setCophylogeny(inputCophylogeny);
		this.inputCophylogenyViewer.initCophylogeny(true, null);
	}

	/**
	 * @return the inputCophylogenyComposite
	 */
	public InputCophylogenyViewer getInputCophylogenyViewer() {
		return inputCophylogenyViewer;
	}

	/**
	 * @param inputCophylogenyViewer
	 *            the inputCophylogenyComposite to set
	 */
	public void setInputCophylogenyViewer(
			InputCophylogenyViewer inputCophylogenyViewer) {
		this.inputCophylogenyViewer = inputCophylogenyViewer;
	}

	public void rotateRight() {
		this.inputCophylogenyViewer.rotateRight();
	}

	public void rotateLeft() {
		this.inputCophylogenyViewer.rotateLeft();
	}

	public void flipVertical() {
		this.inputCophylogenyViewer.flipVertical();
	}

	public void flipHorizontal() {
		this.inputCophylogenyViewer.flipHorizontal();
	}

	/**
	 * @param dirty
	 *            the dirty to set
	 */
	public void setDirty(boolean dirty) {
		if (this.dirty != dirty) {
			this.dirty = dirty;
			this.inputCophylogenyViewer.setDirty(dirty);
			firePropertyChange(IEditorPart.PROP_DIRTY);
		}
	}

}
