/**
 * 
 */
package com.wieseke.cptk.input.operation;

import java.util.List;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;

import com.wieseke.cptk.input.dao.InputHostNode;
import com.wieseke.cptk.input.dao.InputParasiteNode;
import com.wieseke.cptk.input.viewer.InputCophylogenyViewer;

/**
 * @author Nicolas Wieseke
 * 
 */
public class EditDeleteHostAssociationsOperation extends AbstractOperation {

	private InputCophylogenyViewer viewer;
	private InputHostNode node;
	private List<InputParasiteNode> oldAssociations;

	public EditDeleteHostAssociationsOperation(InputCophylogenyViewer viewer, InputHostNode node) {
		super("delete association of node " + node.getLabel());
		this.viewer = viewer;
		this.node = node;
		this.oldAssociations = viewer.getCophylogeny().getAssociationPartners(node,
				viewer.getCophylogeny().getParasiteRoot());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus execute(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		// if (!this.node.isLeaf())
		// return Status.CANCEL_STATUS;
		return redo(monitor, info);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus redo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		for (InputParasiteNode parasiteNode : this.oldAssociations) {
			parasiteNode.getAssociations().remove(this.node);
		}
		this.viewer.updateAssociations();
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse
	 * .core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	
	public IStatus undo(IProgressMonitor monitor, IAdaptable info) throws ExecutionException {
		for (InputParasiteNode parasiteNode : this.oldAssociations) {
			parasiteNode.getAssociations().add(this.node);
		}
		this.viewer.updateAssociations();
		this.viewer.redraw();
		this.viewer.setDirty(true);
		return Status.OK_STATUS;
	}

}
