/**
 * 
 */
package com.wieseke.cptk.input.wizards;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.text.DecimalFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.FocusEvent;
import org.eclipse.swt.events.FocusListener;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Scale;
import org.eclipse.swt.widgets.Text;

import com.wieseke.cptk.common.constants.CostsConstants;
import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.reconstruction.constants.BaseSolverConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ChooseCophylogenyCostsWizardPage extends WizardPage {

	private InputCophylogeny cophylogeny;

	private Group costsGroup;
	private Button absoluteValuesButton;
	private Button probabilisticValuesButton;
	private Group absoluteValuesGroup;
	private Map<String, Text> aTexts;
	private Group probabilisticValuesGroup;
	private Map<String, Button> pButtons;
	private Map<String, Scale> pScales;
	private Map<String, Label> pLabels;

	private BigDecimal transformFactor = new BigDecimal("100000");
	private DecimalFormat transformDecimalFormat = new DecimalFormat("0.00000");
	private boolean useProbabilisticValues;
	private Map<String, BigDecimal> absoluteValues;

	public ChooseCophylogenyCostsWizardPage(String pageName,
			InputCophylogeny cophylogeny) {
		super(pageName);
		this.cophylogeny = cophylogeny;
		this.aTexts = new HashMap<String, Text>();
		this.pButtons = new HashMap<String, Button>();
		this.pScales = new HashMap<String, Scale>();
		this.pLabels = new HashMap<String, Label>();
		this.setTitle(pageName);
		this.setDescription("Choose the event costs.");

		checkProbabilisticValues();
	}

	private void checkProbabilisticValues() {
		// asume absolute values
		useProbabilisticValues = false;
		this.absoluteValues = new HashMap<String, BigDecimal>();
		for (Entry<String, BigDecimal> cost : this.cophylogeny.getCosts()
				.entrySet()) {
			this.absoluteValues.put(cost.getKey(), cost.getValue());
		}

		BigDecimal costSum = BigDecimal.ZERO;
		BigDecimal reciprocalSum = BigDecimal.ZERO;
		// check PROBABILITYCOSTS option set to 1
		if (this.cophylogeny.getOptions().get(
				OptionsConstants.PROBABILITY_COSTS_ID) != null
				&& this.cophylogeny.getOptions().get(
						OptionsConstants.PROBABILITY_COSTS_ID).equals(
						OptionsConstants.PROBABILITY_COSTS_1_ID)) {
			// check if all costs are set to values grater 0
			Map<String, String> costs = CostsConstants.getCosts();
			for (Entry<String, String> cost : costs.entrySet()) {
				if (this.cophylogeny.getCosts().get(cost.getKey()) == null
						|| this.cophylogeny.getCosts().get(cost.getKey())
								.compareTo(BigDecimal.ZERO) <= 0) {
					return;
				} else {
					costSum = costSum.add(this.cophylogeny.getCosts().get(
							cost.getKey()));
					reciprocalSum = reciprocalSum.add(BigDecimal.ONE.divide(
							this.cophylogeny.getCosts().get(cost.getKey()),
							BaseSolverConstants.BIGDECIMAL_SCALE,
							RoundingMode.CEILING));
				}
			}
			// check if sum is 1
			if (costSum.compareTo(BigDecimal.ONE) != 0) {
				return;
			}
		} else {
			return;
		}
		// calculate new absolute values from the reciprocals
		for (Entry<String, BigDecimal> cost : this.cophylogeny.getCosts()
				.entrySet()) {
			this.absoluteValues.remove(cost.getKey());
			this.absoluteValues.put(cost.getKey(), BigDecimal.ONE.divide(
					this.cophylogeny.getCosts().get(cost.getKey()),
					BaseSolverConstants.BIGDECIMAL_SCALE, RoundingMode.CEILING)
					.divide(reciprocalSum,
							BaseSolverConstants.BIGDECIMAL_SCALE,
							RoundingMode.CEILING));
		}
		useProbabilisticValues = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.jface.dialogs.IDialogPage#createControl(org.eclipse.swt.widgets
	 * .Composite)
	 */

	public void createControl(Composite parent) {
		Composite composite = new Composite(parent, SWT.FILL);

		FillLayout compositeLayout = new FillLayout();
		compositeLayout.spacing = 10;
		compositeLayout.marginWidth = 10;
		compositeLayout.marginHeight = 10;
		composite.setLayout(compositeLayout);

		costsGroup = new Group(composite, SWT.NONE);
		GridLayout costsGroupLayout = new GridLayout(2, false);
		costsGroupLayout.horizontalSpacing = 10;
		costsGroupLayout.verticalSpacing = 10;
		costsGroupLayout.marginWidth = 10;
		costsGroupLayout.marginHeight = 10;
		costsGroupLayout.marginTop = 10;
		costsGroupLayout.marginBottom = 10;
		costsGroupLayout.marginLeft = 10;
		costsGroupLayout.marginRight = 10;
		costsGroup.setLayout(costsGroupLayout);
		costsGroup.setText("Costs");

		absoluteValuesButton = new Button(costsGroup, SWT.RADIO | SWT.LEFT);
		absoluteValuesButton.setText("Use absolute Values");

		probabilisticValuesButton = new Button(costsGroup, SWT.RADIO | SWT.LEFT);
		probabilisticValuesButton.setText("Use probabilistic Values");

		absoluteValuesGroup = new Group(costsGroup, SWT.NONE);
		GridLayout absoluteValuesGroupGridLayout = new GridLayout(2, false);
		absoluteValuesGroupGridLayout.horizontalSpacing = 10;
		absoluteValuesGroupGridLayout.verticalSpacing = 10;
		absoluteValuesGroupGridLayout.marginWidth = 10;
		absoluteValuesGroupGridLayout.marginHeight = 10;
		absoluteValuesGroupGridLayout.marginTop = 10;
		absoluteValuesGroupGridLayout.marginBottom = 10;
		absoluteValuesGroupGridLayout.marginLeft = 10;
		absoluteValuesGroupGridLayout.marginRight = 10;
		absoluteValuesGroup.setLayout(absoluteValuesGroupGridLayout);
		GridData absoluteValuesGroupLayoutData = new GridData();
		absoluteValuesGroupLayoutData.verticalAlignment = GridData.FILL;
		absoluteValuesGroup.setLayoutData(absoluteValuesGroupLayoutData);
		absoluteValuesGroup.setText("absolute Values");

		Map<String, String> costs = CostsConstants.getCosts();

		for (Entry<String, String> entry : costs.entrySet()) {
			Label label = new Label(absoluteValuesGroup, SWT.NONE);
			label.setText(entry.getKey());
			label.setToolTipText(entry.getValue());

			final Text aText = new Text(absoluteValuesGroup, SWT.BORDER);
			aText.addFocusListener(new FocusListener() {

				public void focusGained(FocusEvent e) {
					Text textWidget = (Text) e.widget;
					textWidget.setSelection(0, textWidget.getText().length());
				}

				public void focusLost(FocusEvent e) {
					// TODO Auto-generated method stub

				}
			});
			GridData absoluteValuesSpinnerData = new GridData();
			absoluteValuesSpinnerData.widthHint = 100;
			absoluteValuesSpinnerData.heightHint = 20;
			absoluteValuesSpinnerData.horizontalAlignment = GridData.FILL;
			absoluteValuesSpinnerData.grabExcessHorizontalSpace = true;
			aText.setLayoutData(absoluteValuesSpinnerData);
			BigDecimal value = this.absoluteValues.get(entry.getKey());
			aText.setText(value.toString());
			aText.setTextLimit(BaseSolverConstants.BIGDECIMAL_SCALE + 2);
			this.aTexts.put(entry.getKey(), aText);
		}

		probabilisticValuesGroup = new Group(costsGroup, SWT.NONE);
		GridLayout probabilisticValuesGroupGridLayout = new GridLayout(4, false);
		probabilisticValuesGroupGridLayout.horizontalSpacing = 10;
		probabilisticValuesGroupGridLayout.verticalSpacing = 10;
		probabilisticValuesGroupGridLayout.marginWidth = 10;
		probabilisticValuesGroupGridLayout.marginHeight = 10;
		probabilisticValuesGroupGridLayout.marginTop = 10;
		probabilisticValuesGroupGridLayout.marginBottom = 10;
		probabilisticValuesGroupGridLayout.marginLeft = 10;
		probabilisticValuesGroupGridLayout.marginRight = 10;
		probabilisticValuesGroup.setLayout(probabilisticValuesGroupGridLayout);
		GridData probabilisticValuesGroupData = new GridData();
		probabilisticValuesGroupData.minimumWidth = 400;
		probabilisticValuesGroupData.grabExcessHorizontalSpace = true;
		probabilisticValuesGroupData.horizontalAlignment = GridData.FILL;
		probabilisticValuesGroupData.verticalAlignment = GridData.FILL;
		probabilisticValuesGroup.setLayoutData(probabilisticValuesGroupData);
		probabilisticValuesGroup.setText("probabilistic Values");

		int index = 0;
		for (Entry<String, String> entry : costs.entrySet()) {
			Label label = new Label(probabilisticValuesGroup, SWT.NONE);
			label.setText(entry.getKey());
			label.setToolTipText(entry.getValue());

			Button pButton = new Button(probabilisticValuesGroup, SWT.CHECK
					| SWT.LEFT);
			pButton.setText("");
			this.pButtons.put(entry.getKey(), pButton);

			Scale pScale = new Scale(probabilisticValuesGroup, SWT.NONE);
			GridData probabilisticValuesScaleData = new GridData();
			probabilisticValuesScaleData.widthHint = 250;
			probabilisticValuesScaleData.minimumWidth = 100;
			probabilisticValuesScaleData.grabExcessHorizontalSpace = true;
			probabilisticValuesScaleData.horizontalAlignment = GridData.FILL;
			pScale.setLayoutData(probabilisticValuesScaleData);
			pScale.setMinimum(1);
			pScale.setMaximum(99997);
			pScale.setIncrement(1);
			pScale.setPageIncrement(10000);
			pScale.setData(entry.getKey());
			this.pScales.put(entry.getKey(), pScale);

			Label pLabel = new Label(probabilisticValuesGroup, SWT.NONE);
			if (this.useProbabilisticValues) {
				BigDecimal value = this.cophylogeny.getCosts().get(
						entry.getKey());
				pScale.setSelection(transformToControlValue(value));
				pLabel.setText(this.transformDecimalFormat.format(value));
			} else {
				pLabel.setText("");
			}
			GridData probabilisticValuesLabelGridData = new GridData();
			probabilisticValuesLabelGridData.widthHint = 60;
			probabilisticValuesLabelGridData.heightHint = 30;
			pLabel.setLayoutData(probabilisticValuesLabelGridData);
			this.pLabels.put(entry.getKey(), pLabel);

			pScale.addSelectionListener(new SelectionListener() {

				public void widgetDefaultSelected(SelectionEvent e) {
				}

				public void widgetSelected(SelectionEvent e) {
					String key = e.widget.getData().toString();
					Scale scale = pScales.get(key);
					// Button button = pButtons.get(key);
					Label label = pLabels.get(key);
					int fixedCount = 0;
					int unfixedCount = 0;
					int fixedSum = 0;
					int unfixedSum = 0;
					for (Entry<String, Button> entry : pButtons.entrySet()) {
						if (!entry.getKey().equals(key)) {
							if (!entry.getValue().getSelection()) {
								unfixedCount++;
								unfixedSum += pScales.get(entry.getKey())
										.getSelection();
							} else {
								fixedCount++;
								fixedSum += pScales.get(entry.getKey())
										.getSelection();
							}
						}
					}
					// no degrees of freedom -> set value
					if (unfixedCount == 0) {
						scale.setSelection(100000 - fixedSum);
						label.setText(transformDecimalFormat
								.format(transformFromControlValue(scale
										.getSelection())));
					}
					// maximum reached -> set this value to maximum
					// and all unfixed values to minimum
					else if (100000 - fixedSum - unfixedCount < scale
							.getSelection()) {
						scale.setSelection(100000 - fixedSum - unfixedCount);
						label.setText(transformDecimalFormat
								.format(transformFromControlValue(scale
										.getSelection())));
						for (Entry<String, Label> entry : pLabels.entrySet()) {
							if (!entry.getKey().equals(key)
									&& !pButtons.get(entry.getKey())
											.getSelection()) {
								pScales.get(entry.getKey()).setSelection(1);
								entry
										.getValue()
										.setText(
												transformDecimalFormat
														.format(transformFromControlValue(pScales
																.get(
																		entry
																				.getKey())
																.getSelection())));
							}
						}
						// calculate new for unfixed values
					} else {
						label.setText(transformDecimalFormat
								.format(transformFromControlValue(scale
										.getSelection())));

						int difference = 100000 - fixedSum - unfixedSum
								- scale.getSelection();
						int step = (difference < 0) ? -1 : 1;
						boolean changed = true;
						while (difference != 0 && changed) {
							changed = false;
							for (Entry<String, Label> entry : pLabels
									.entrySet()) {
								if (!entry.getKey().equals(key)) {
									if (!pButtons.get(entry.getKey())
											.getSelection()) {
										if (pScales.get(entry.getKey())
												.getSelection()
												+ step >= pScales.get(
												entry.getKey()).getMinimum()
												&& pScales.get(entry.getKey())
														.getSelection()
														+ step <= pScales.get(
														entry.getKey())
														.getMaximum()) {

											pScales
													.get(entry.getKey())
													.setSelection(
															pScales
																	.get(
																			entry
																					.getKey())
																	.getSelection()
																	+ step);
											changed = true;
											difference -= step;
											if (difference == 0)
												break;
										}
									}
								}
							}
						}
						for (Entry<String, Label> entry : pLabels.entrySet()) {
							entry
									.getValue()
									.setText(
											transformDecimalFormat
													.format(transformFromControlValue(pScales
															.get(entry.getKey())
															.getSelection())));
						}
					}
				}

			});

			index++;
		}
		absoluteValuesButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				boolean enabled = absoluteValuesButton.getSelection();
				for (Control child : absoluteValuesGroup.getChildren()) {
					child.setEnabled(enabled);
				}
				absoluteValuesGroup.setEnabled(enabled);
			}
		});
		probabilisticValuesButton.addSelectionListener(new SelectionListener() {

			public void widgetDefaultSelected(SelectionEvent e) {
			}

			public void widgetSelected(SelectionEvent e) {
				boolean enabled = probabilisticValuesButton.getSelection();
				for (Control child : probabilisticValuesGroup.getChildren()) {
					child.setEnabled(enabled);
				}
				probabilisticValuesGroup.setEnabled(enabled);
			}
		});

		if (this.useProbabilisticValues) {
			absoluteValuesButton.setSelection(false);
			probabilisticValuesButton.setSelection(true);
			for (Control child : absoluteValuesGroup.getChildren()) {
				child.setEnabled(false);
			}
			absoluteValuesGroup.setEnabled(false);
			probabilisticValuesGroup.setEnabled(true);
		} else {
			probabilisticValuesButton.setSelection(false);
			absoluteValuesButton.setSelection(true);
			for (Control child : probabilisticValuesGroup.getChildren()) {
				child.setEnabled(false);
			}
			probabilisticValuesGroup.setEnabled(false);
			absoluteValuesGroup.setEnabled(true);
		}

		composite.layout();

		setControl(composite);
	}

	private int transformToControlValue(BigDecimal value) {
		if (value == null)
			return 0;
		return value.multiply(this.transformFactor).intValue();
	}

	public BigDecimal transformFromControlValue(int value) {
		return new BigDecimal(value).divide(this.transformFactor,
				BaseSolverConstants.BIGDECIMAL_SCALE, RoundingMode.CEILING);
	}

	/**
	 * @return the aTexts
	 */
	public Map<String, Text> getATexts() {
		return aTexts;
	}

	/**
	 * @return the pScales
	 */
	public Map<String, Scale> getPScales() {
		return pScales;
	}

	public boolean useProbabilisticValues() {
		return this.probabilisticValuesButton.getSelection();
	}
}
