package com.wieseke.cptk.main;

import java.util.Map;
import java.util.TreeMap;

import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.output.dao.OutputHostNode;
import com.wieseke.cptk.output.dao.OutputParasiteNode;
import com.wieseke.cptk.reconstruction.dao.EventType;
import com.wieseke.cptk.reconstruction.dao.ReconstructionChildMapping;

public class BootstrappingHelper {

	private long assocFrequencies[][];
	private OutputCophylogeny outputCophylogeny;
	private long numOfAssociations;
	private long numOfReconstructions;
	private double[][] eventFrequenciesPerHost;
	public long bestReconstruction;

	public BootstrappingHelper(OutputCophylogeny outputCophylogeny) {
		this.outputCophylogeny = outputCophylogeny;
		this.assocFrequencies = new long[outputCophylogeny.getHostRoot()
				.getChildrenCount() + 1][outputCophylogeny.getParasiteRoot()
				.getChildrenCount() + 1];
	}

	public long[][] computeAssociationFrequencies() {
		this.numOfReconstructions = 0;
		long minNumOfAssociations = Long.MAX_VALUE;
		outputCophylogeny.chooseReconstruction(0);
		do {
			// EventType events = outputCophylogeny.getSolution().getEvents();
			// if (events.getCospeciations() > events.getHostswitches()) {
			if (numOfReconstructions % 1000000 == 0) {
				System.out.println(numOfReconstructions);
			}
			numOfAssociations = 0;
			computeAssociationFrequenciesForParasite(outputCophylogeny
					.getParasiteRoot());
			if (numOfAssociations < minNumOfAssociations) {
				minNumOfAssociations = numOfAssociations;
				this.bestReconstruction = numOfReconstructions;
			}
			// }
			this.numOfReconstructions++;
		} while (outputCophylogeny.nextReconstruction(false, false));
		System.out.println("association frequencies computed for "
				+ this.numOfReconstructions + " Reconstructions");
		return this.assocFrequencies;
	}

	public Map<Double, Long> computeEventsFrequencies() {
		long reconstruction = 0;
		Map<Double, Long> values = new TreeMap<Double, Long>();
		int numHostNodes = outputCophylogeny.getHostRoot().getChildrenCount() + 1;
		double ratio = 1d / (double) numHostNodes;
		do {
			EventType events = outputCophylogeny.getParasiteRoot()
					.getDerivedEvents();
			if (events.getCospeciations() > 0
					&& events.getCospeciations() > events.getHostswitches()) {
				this.eventFrequenciesPerHost = new double[numHostNodes][4];
				if (reconstruction % 1000000 == 0) {
					System.out.println(reconstruction);
				}
				double freqSum[] = { events.getCospeciations(),
						events.getSortings(), events.getDuplications(),
						events.getHostswitches() };
				computeEventFrequenciesForParasite(outputCophylogeny
						.getParasiteRoot());
				double value = 0d;
				for (double freq[] : this.eventFrequenciesPerHost) {
					for (int i = 0; i < 4; i++) {
						if (freqSum[i] != 0) {
							value += Math.abs(ratio - (freq[i] / freqSum[i]));
						} else {
							// value += ratio;
						}
					}
				}
				values.put(value, reconstruction);
			}
			reconstruction++;
		} while (outputCophylogeny.nextReconstruction(false, false));
		System.out.println("event frequencies computed for " + reconstruction
				+ " Reconstructions");
		return values;
	}

	private void computeEventFrequenciesForParasite(OutputParasiteNode node) {
		for (OutputParasiteNode child : node.getChildren()) {
			computeEventFrequenciesForParasite(child);
		}

		OutputHostNode tempAssociationHostNode = node.getAssociation();
		OutputParasiteNode parent = node.getParent();

		if (!node.isLeaf()) {
			// Cospeciation
			if (node.isCospeciation(false)) {
				this.eventFrequenciesPerHost[tempAssociationHostNode
						.getNumber()][0]++;
			}

			// Duplication
			else {
				this.eventFrequenciesPerHost[tempAssociationHostNode
						.getNumber()][2]++;
			}
		}

		if (parent != null) {
			ReconstructionChildMapping mapping = node.getMapping();
			OutputHostNode parentAssociationHostNode = parent.getAssociation();
			if (mapping.isHostswitch()) {
				while (tempAssociationHostNode.getNumber() > node
						.getLandingNode().getNumber()) {
					// Sortings after Landing (but not at deepest associated
					// host)
					if (tempAssociationHostNode.getNumber() != node
							.getAssociation().getNumber()) {
						this.eventFrequenciesPerHost[tempAssociationHostNode
								.getNumber()][1]++;
					}
					tempAssociationHostNode = tempAssociationHostNode
							.getParent();
					if (tempAssociationHostNode == null) {
						throw new RuntimeException();
					}
				}
				// Hostswitch Landing part
				this.eventFrequenciesPerHost[tempAssociationHostNode
						.getNumber()][3] += 0.5d;
				tempAssociationHostNode = node.getTakeofNode();
				// Hostswitch Takeof part
				this.eventFrequenciesPerHost[tempAssociationHostNode
						.getNumber()][3] += 0.5d;
			}
			tempAssociationHostNode = tempAssociationHostNode.getParent();
			if (tempAssociationHostNode != null) {
				while (tempAssociationHostNode.getNumber() > parentAssociationHostNode
						.getNumber()) {
					// Sorting
					this.eventFrequenciesPerHost[tempAssociationHostNode
							.getNumber()][1]++;
					tempAssociationHostNode = tempAssociationHostNode
							.getParent();
					if (tempAssociationHostNode == null) {
						throw new RuntimeException();
					}
				}
				if (tempAssociationHostNode.getNumber() == parentAssociationHostNode
						.getNumber() && !parent.isCospeciation(true)) {
					// Sorting
					this.eventFrequenciesPerHost[tempAssociationHostNode
							.getNumber()][1]++;
				}
			}
		}

	}

	private void computeAssociationFrequenciesForParasite(
			OutputParasiteNode node) {
		for (OutputParasiteNode child : node.getChildren()) {
			computeAssociationFrequenciesForParasite(child);
		}
		OutputHostNode tempAssociationHostNode = node.getAssociation();
		OutputParasiteNode parent = node.getParent();
		if (parent != null) {
			ReconstructionChildMapping mapping = node.getMapping();
			OutputHostNode parentAssociationHostNode = parent.getAssociation();
			if (mapping.isHostswitch()) {
				while (tempAssociationHostNode.getNumber() >= node
						.getLandingNode().getNumber()) {
					this.assocFrequencies[tempAssociationHostNode.getNumber()][node
							.getNumber()]++;
					numOfAssociations++;
					tempAssociationHostNode = tempAssociationHostNode
							.getParent();
					if (tempAssociationHostNode == null) {
						throw new RuntimeException();
					}
				}
				tempAssociationHostNode = node.getTakeofNode();
				if (tempAssociationHostNode.getNumber() == parentAssociationHostNode
						.getNumber()) {
					return;
				}
			}
			while (tempAssociationHostNode != null
					&& tempAssociationHostNode.getNumber() > parentAssociationHostNode
							.getNumber()) {
				this.assocFrequencies[tempAssociationHostNode.getNumber()][node
						.getNumber()]++;
				numOfAssociations++;
				tempAssociationHostNode = tempAssociationHostNode.getParent();
				if (tempAssociationHostNode == null) {
					throw new RuntimeException();
				}
			}
			if (!parent.isCospeciation(true)) {
				this.assocFrequencies[tempAssociationHostNode.getNumber()][node
						.getNumber()]++;
				numOfAssociations++;
			}
		} else {
			this.assocFrequencies[tempAssociationHostNode.getNumber()][node
					.getNumber()]++;
			numOfAssociations++;
		}

	}

	public Map<Double, Long> rateReconstructions() {
		long currentReconstruction = this.numOfReconstructions - 1;
		this.bestReconstruction = currentReconstruction;
		double bestValue = 0d;
		System.out.println(this.numOfReconstructions / Integer.MAX_VALUE);
		Map<Double, Long> values = new TreeMap<Double, Long>();
		do {
			if (currentReconstruction % 1000000 == 0) {
				System.out.println(currentReconstruction);
			}
			numOfAssociations = 0;
			long result = computeReconstructionRateForParasite(
					outputCophylogeny.getParasiteRoot(), true);
			double value = (double) result / (double) numOfAssociations;
			values.put(value, currentReconstruction);
			// EventType events = outputCophylogeny.getSolution().getEvents();
			// if (events.getCospeciations() > events.getHostswitches()) {
			if (value > bestValue) {
				bestValue = value;
				this.bestReconstruction = currentReconstruction;
			}
			// }
			currentReconstruction--;
		} while (outputCophylogeny.previousReconstruction(false));
		System.out.println("reconstructions rated");
		return values;
	}

	private long computeReconstructionRateForParasite(OutputParasiteNode node,
			boolean ignoreLeaves) {
		long result = 0;
		for (OutputParasiteNode child : node.getChildren()) {
			result += computeReconstructionRateForParasite(child, ignoreLeaves);
		}
		OutputHostNode tempAssociationHostNode = node.getAssociation();
		OutputParasiteNode parent = node.getParent();
		boolean leafDone = false;
		if (parent != null) {
			ReconstructionChildMapping mapping = node.getMapping();
			OutputHostNode parentAssociationHostNode = parent.getAssociation();
			if (mapping.isHostswitch()) {
				while (tempAssociationHostNode.getNumber() >= node
						.getLandingNode().getNumber()) {
					if (!node.isLeaf() || !ignoreLeaves || !leafDone) {
						result += this.assocFrequencies[tempAssociationHostNode
								.getNumber()][node.getNumber()];
						numOfAssociations++;
						leafDone = true;
					}
					tempAssociationHostNode = tempAssociationHostNode
							.getParent();
					if (tempAssociationHostNode == null) {
						throw new RuntimeException();
					}
				}
				tempAssociationHostNode = node.getTakeofNode();
				if (tempAssociationHostNode.getNumber() == parentAssociationHostNode
						.getNumber()) {
					return result;
				}
			}
			while (tempAssociationHostNode != null
					&& tempAssociationHostNode.getNumber() > parentAssociationHostNode
							.getNumber()) {
				if (!node.isLeaf() || !ignoreLeaves || !leafDone) {
					result += this.assocFrequencies[tempAssociationHostNode
							.getNumber()][node.getNumber()];
					numOfAssociations++;
					leafDone = true;
				}
				tempAssociationHostNode = tempAssociationHostNode.getParent();
				if (tempAssociationHostNode == null) {
					throw new RuntimeException();
				}
			}
			if (!parent.isCospeciation(true)) {
				if (!node.isLeaf() || !ignoreLeaves || !leafDone) {
					result += this.assocFrequencies[tempAssociationHostNode
							.getNumber()][node.getNumber()];
					numOfAssociations++;
					leafDone = true;
				}
			}
		} else {
			if (!node.isLeaf() || !ignoreLeaves || !leafDone) {
				result += this.assocFrequencies[tempAssociationHostNode
						.getNumber()][node.getNumber()];
				numOfAssociations++;
				leafDone = true;
			}
		}
		return result;

	}

}
