/**
 * 
 */
package com.wieseke.cptk.main;

import java.io.File;
import java.io.FileInputStream;
import java.io.OutputStreamWriter;
import java.util.Iterator;
import java.util.TreeMap;

import org.apache.commons.io.FileUtils;

import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.input.format.nexus.NexusDeserializer;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionSolution;

/**
 * @author Nicolas Wieseke
 * 
 */
public class ViroBonnReconstruction {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		if (args.length == 0 || args[0].equalsIgnoreCase("-h")
				|| args[0].equalsIgnoreCase("-help")) {
			printHelp();
			return;
		}
		try {
			String filename = args[0];
			NexusDeserializer nd = new NexusDeserializer();
			InputCophylogeny cophylogeny = nd.deserialize(new FileInputStream(
					filename), filename);
			// cophylogeny.deleteNoneLeafAssociations(cophylogeny.getParasiteRoot());
			ReconstructionCophylogeny reconstructionCophylogeny = new ReconstructionCophylogeny(
					cophylogeny);

			// OutputStreamWriter oswPALog = new OutputStreamWriter(
			// FileUtils.openOutputStream(new File(filename + ".pa.log")));
			// oswPALog.write("Counter\tQuality\tTotalCost\tTotalEvents\tCoCost\tCoEvents\tCoProb\tCoFreq\tSoCost\tSoEvents\tSoProb\tSoFreq\tDuCost\tDuEvents\tDuProb\tDuFreq\tHsCost\tHsEvents\tHsProb\tHsFreq\tRootMapping\n");
			// reconstructionCophylogeny.compute(oswPALog);
			// oswPALog.close();
			long time = System.currentTimeMillis();
			reconstructionCophylogeny.compute(null);
			time = System.currentTimeMillis() - time;
			OutputCophylogeny outputCophylogeny = new OutputCophylogeny(
					reconstructionCophylogeny);
			ReconstructionHelper rh = new ReconstructionHelper(
					outputCophylogeny);
			OutputStreamWriter oswPaReco = new OutputStreamWriter(
					FileUtils.openOutputStream(new File(filename + ".vb.reco")));
			OutputStreamWriter oswPaCosp = new OutputStreamWriter(
					FileUtils.openOutputStream(new File(filename + ".vb.cosp")));
			// oswPaReco.write("Duration in milliseconds:\n" + time + "\n\n");
			// rh.computeAssociationListsViroBonn(oswPaReco);
			// rh.computeAssociationLists(oswPaReco, oswPaCosp);

			Iterator<ReconstructionSolution> iter = reconstructionCophylogeny
					.getBestSolutions().values().iterator();
			TreeMap<Double, ReconstructionSolution> solutions = new TreeMap<Double, ReconstructionSolution>();
			while (iter.hasNext()) {
				ReconstructionSolution solution = iter.next();
				solutions.put(solution.getQuality().doubleValue(), solution);
			}
			iter = solutions.values().iterator();
			boolean above25 = false;
			while (iter.hasNext()) {
				ReconstructionSolution solution = iter.next();
				// if (!above25
				// && solution.getQuality()
				// .compareTo(new BigDecimal(0.25)) > 0) {
				// above25 = true;
				// oswPaReco.close();
				// oswPaReco = new OutputStreamWriter(
				// FileUtils.openOutputStream(new File(filename
				// + ".vb.reco.50")));
				// oswPaCosp.close();
				// oswPaCosp = new OutputStreamWriter(
				// FileUtils.openOutputStream(new File(filename
				// + ".vb.cosp.50")));
				// }
				// if (solution.getQuality().compareTo(new BigDecimal(0.5)) > 0)
				// {
				// break;
				// }
				ReconstructionCosts costs = solution.getCosts();
				reconstructionCophylogeny.applyCosts(costs);
				reconstructionCophylogeny.getOptions().put(
						OptionsConstants.AUTOMATIC_COSTS_ID,
						OptionsConstants.AUTOMATIC_COSTS_0_ID);
				reconstructionCophylogeny.compute(null);
				outputCophylogeny = new OutputCophylogeny(
						reconstructionCophylogeny);
				rh = new ReconstructionHelper(outputCophylogeny);
				// rh.computeAssociationListsViroBonn(oswPaReco);
				rh.computeAssociationLists(oswPaReco, oswPaCosp);
			}

			oswPaReco.close();
			oswPaCosp.close();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * prints the help information
	 * */
	static public void printHelp() {
		System.out
				.println("\nSyntax:\n"
						+ "java -cp .:./* com.wieseke.cptk.main.Reconstruction [filename.nex]"
						+ "\n\nFor the java -cp parameter use ';' as classpath separanor on Windows machines (not ':').");
	}
}
