/**
 * 
 */
package com.wieseke.cptk.output.dao;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.api.IHostNode;
import com.wieseke.cptk.common.api.IPositionHostNode;
import com.wieseke.cptk.common.util.Tupel;
import com.wieseke.cptk.input.preferences.InputPreferenceConstants;
import com.wieseke.cptk.output.preferences.OutputPreferenceConstants;

/**
 * @author Nicolas Wieseke
 * 
 */
public class OutputHostNode extends OutputNode implements IPositionHostNode {

	public OutputHostNode(IHostNode node) {
		super(node);
		this.setRank(node.getRank());
		this.parasiteShifts = new ArrayList<Integer>();
	}

	
	public OutputHostNode getParent() {
		return (OutputHostNode) super.getParent();
	}

	@SuppressWarnings("unchecked")
	
	public List<OutputHostNode> getChildren() {
		return (List<OutputHostNode>) super.getChildren();
	}

	private List<Integer> parasiteShifts;
	private int minParasiteShift;

	/**
	 * the timezone of the host node
	 */
	public int rank;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IHostNode#getRank()
	 */
	
	public int getRank() {
		return rank;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IHostNode#setRank(int)
	 */
	
	public void setRank(int rank) {
		this.rank = rank;
	}

	/**
	 * @return the parasiteShifts
	 */
	public List<Integer> getParasiteShifts() {
		return parasiteShifts;
	}

	/**
	 * @param parasiteShifts
	 *            the parasiteShifts to set
	 */
	public void setParasiteShifts(List<Integer> parasiteShifts) {
		this.parasiteShifts = parasiteShifts;
	}

	/**
	 * @return the minParasiteShift
	 */
	public int getMinParasiteShift() {
		return minParasiteShift;
	}

	/**
	 * @param minParasiteShift
	 *            the minParasiteShift to set
	 */
	public void setMinParasiteShift(int minParasiteShift) {
		this.minParasiteShift = minParasiteShift;
	}

	
	public String getToolTipText() {
		return getLabel() + " (" + rank + ")";
	}

	
	public String getRankText() {
		return String.valueOf(rank);
	}

	
	public Tupel<Integer, Integer> calcNodePositions(int nextLeafPosX) {
		this.parasiteShifts = new ArrayList<Integer>();
		this.minParasiteShift = 0;
		parasiteShifts.add(0);
		IPreferenceStore store = PlatformUI.getPreferenceStore();
		int xDistance = store
				.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_REARRANGE_NODES_XDISTANCE);
		int yDistance = store
				.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_REARRANGE_NODES_YDISTANCE);
		String linetype = store
				.getString(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_LINETYPE);

		int nextPosY = 0;
		if (isLeaf()) {
			setPosX(nextLeafPosX * xDistance);
			nextLeafPosX++;
			nextPosY++;
			setPosY(nextPosY * yDistance);
		} else {
			for (OutputNode node : getChildren()) {
				Tupel<Integer, Integer> integerTupel = ((OutputNode) node)
						.calcNodePositions(nextLeafPosX);
				nextLeafPosX = integerTupel.getValue1();
				nextPosY = Math.max(nextPosY, integerTupel.getValue2());
			}

			OutputNode leftLeaf;
			int leftChildPosX;
			OutputNode rightLeaf;
			int rightChildPosX;

			// diagonal lines
			if (linetype
					.equals(InputPreferenceConstants.INPUT_VIEWER_LINETYPE_ACUTE_ANGLED)) {
				leftLeaf = this;
				rightLeaf = this;
				while (!leftLeaf.isLeaf()) {
					leftLeaf = leftLeaf.getChildren().get(0);
				}
				while (!rightLeaf.isLeaf()) {
					rightLeaf = rightLeaf.getChildren().get(
							rightLeaf.getDirectChildrenCount() - 1);
				}
				int minPosY = yDistance / 2;
				for (OutputHostNode child : this.getChildren()) {
					minPosY = Math
							.max(minPosY, child.getPosY() + yDistance / 2);
				}

				leftChildPosX = leftLeaf.getPosX();
				rightChildPosX = rightLeaf.getPosX();

				setPosX((rightChildPosX + leftChildPosX) / 2);
				setPosY(Math.max(minPosY, yDistance / 2
						* Math.abs(rightChildPosX - leftChildPosX) / xDistance
						+ yDistance));
			}
			// orthogonal lines
			else {
				// set y position of the child node as high as possible (except
				// for the leaf nodes)
				// for (InputNode node : getChildren()) {
				// if (!node.isLeaf())
				// node.setPosY(nextPosY * yDistance);
				// }

				leftLeaf = getChildren().get(0);
				rightLeaf = getChildren().get(getChildren().size() - 1);

				leftChildPosX = leftLeaf.getPosX();
				rightChildPosX = rightLeaf.getPosX();

				nextPosY++;

				setPosX((rightChildPosX + leftChildPosX) / 2);
				setPosY(nextPosY * yDistance);
			}

		}
		return new Tupel<Integer, Integer>(nextLeafPosX, nextPosY);
	}
}
