/**
 * 
 */
package com.wieseke.cptk.output.dao;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.jface.preference.IPreferenceStore;
import org.eclipse.swt.graphics.Point;
import org.eclipse.ui.PlatformUI;

import com.wieseke.cptk.common.api.IPositionParasiteNode;
import com.wieseke.cptk.common.dao.ParasiteRank;
import com.wieseke.cptk.common.log.CophylogenyLogger;
import com.wieseke.cptk.common.util.Tupel;
import com.wieseke.cptk.output.preferences.OutputPreferenceConstants;
import com.wieseke.cptk.reconstruction.dao.EventType;
import com.wieseke.cptk.reconstruction.dao.ReconstructionChildMapping;
import com.wieseke.cptk.reconstruction.dao.ReconstructionElement;
import com.wieseke.cptk.reconstruction.dao.ReconstructionInfo;
import com.wieseke.cptk.reconstruction.dao.ReconstructionParasiteNode;

/**
 * @author Nicolas Wieseke
 * 
 */
public class OutputParasiteNode extends OutputNode implements
		IPositionParasiteNode {

	private OutputHostNode[] hostNodes;
	/**
	 * the complete reconstruction info for this node
	 */
	private ReconstructionInfo[] reconstructionInfo;
	/**
	 * the index of this node in the parents child list
	 */
	private int ownChildIndex;
	/**
	 * the index of the currently selected host node and the index for the used
	 * reconstructionInfo
	 */
	private int hostIndex;
	/**
	 * the current ReconstructionIndex
	 */
	private int reconstructionIndex;

	/**
	 * the current mapping
	 */
	private ReconstructionChildMapping mapping;

	/**
	 * the index take-of host node, if this parasite is emerged by a host switch
	 */
	private int takeofIndex;
	/**
	 * the index landing host node, if this parasite is emerged by a host switch
	 */
	private int landingIndex;

	private int xShift;
	private int xShiftDistance;
	private int yShift;
	private int yShiftDistance;
	private List<Point> edgePositions;

	/**
	 * timezone information for the parasite node
	 */
	private ParasiteRank rank;

	/**
	 * the host nodes to which this parasite is associated with.
	 */
	private List<OutputHostNode> associations;

	public OutputParasiteNode(ReconstructionParasiteNode node, int ownChildIndex) {
		super(node);
		this.ownChildIndex = ownChildIndex;
		this.setRank(node.getRank());
		this.associations = new ArrayList<OutputHostNode>();
		this.reconstructionIndex = 0;
		this.takeofIndex = 0;
		this.landingIndex = 0;
		this.reconstructionInfo = node.getReconstructionInfo();
		this.edgePositions = new ArrayList<Point>();

		// TODO: change this somehow to work with command line without throwing
		// an exception
		// try {
		// IPreferenceStore store = PlatformUI.getPreferenceStore();
		// this.xShiftDistance = store
		// .getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_SHIFT_XDISTANCE);
		// this.yShiftDistance = store
		// .getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_SHIFT_YDISTANCE);
		// } catch (Exception e) {
		// }
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.dao.PositionNode#getParent()
	 */

	public OutputParasiteNode getParent() {
		return (OutputParasiteNode) super.getParent();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.output.dao.OutputNode#getChildren()
	 */
	@SuppressWarnings("unchecked")
	public List<OutputParasiteNode> getChildren() {
		return (List<OutputParasiteNode>) super.getChildren();
	}

	private int getShiftX() {
		if (getShiftY() != 0 && getAssociation().getParent() != null) {
			double divX = getAssociation().getParent().getPosX()
					- getAssociation().getPosX();
			double divY = getAssociation().getParent().getPosY()
					- getAssociation().getPosY();
			return xShift * xShiftDistance
					+ (int) Math.round(getShiftY() * divX / divY);
		} else {
			return xShift * xShiftDistance;
		}
	}

	private int getShiftY() {
		return yShift * yShiftDistance;
	}

	private int getShiftX(int xShift, int yShift, OutputHostNode association) {
		if (yShift != 0 && association.getParent() != null) {
			double divX = association.getParent().getPosX()
					- association.getPosX();
			double divY = association.getParent().getPosY()
					- association.getPosY();
			return xShift * xShiftDistance
					+ (int) Math.round(getShiftY(yShift) * divX / divY);
		} else {
			return xShift * xShiftDistance;
		}
	}

	private int getShiftY(int yShift) {
		return yShift * yShiftDistance;
	}

	/**
	 * @return the edgePositions
	 */
	public List<Point> getEdgePositions() {
		return edgePositions;
	}

	/**
	 * @param edgePositions
	 *            the edgePositions to set
	 */
	public void setEdgePositions(List<Point> edgePositions) {
		this.edgePositions = edgePositions;
	}

	/**
	 * @return the rank
	 */

	public ParasiteRank getRank() {
		return rank;
	}

	/**
	 * @param rank
	 *            the rank to set
	 */

	public void setRank(ParasiteRank rank) {
		this.rank = rank;
	}

	/**
	 * @return the ownChildIndex
	 */
	public int getOwnChildIndex() {
		return ownChildIndex;
	}

	/**
	 * @param ownChildIndex
	 *            the ownChildIndex to set
	 */
	public void setOwnChildIndex(int ownChildIndex) {
		this.ownChildIndex = ownChildIndex;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionParasiteNode#getAssociations()
	 */
	public List<OutputHostNode> getAssociations() {
		return associations;
	}

	public OutputHostNode getAssociation() {
		return this.hostNodes[this.hostIndex];
	}

	public ReconstructionElement getReconstruction() {
		return this.reconstructionInfo[this.hostIndex].getReconstructions()
				.get(this.reconstructionIndex);
	}

	public List<ReconstructionElement> getReconstructions() {
		return this.reconstructionInfo[this.hostIndex].getReconstructions();
	}

	/**
	 * @param hostNodes
	 *            the hostNodes to set
	 */
	public void setHostNodes(OutputHostNode[] hostNodes) {
		this.hostNodes = hostNodes;
	}

	/**
	 * @return the hostIndex
	 */
	public int getHostIndex() {
		return hostIndex;
	}

	/**
	 * @param hostIndex
	 *            the hostIndex to set
	 */
	public void setHostIndex(int hostIndex) {
		this.hostIndex = hostIndex;
		this.associations.clear();
		this.associations.add(this.hostNodes[hostIndex]);
	}

	public EventType getDerivedEvents() {
		ReconstructionElement reconstruction = getReconstruction();
		if (reconstruction != null) {
			EventType derivedEvents = new EventType(
					reconstruction.getDirectEvents());
			for (OutputParasiteNode child : getChildren()) {
				derivedEvents = derivedEvents.add(child.getDerivedEvents());
			}
			return derivedEvents;
		}
		return null;
	}

	public EventType getDirectEvents() {
		ReconstructionElement reconstruction = getReconstruction();
		if (reconstruction != null) {
			return reconstruction.getDirectEvents();
		}
		return null;
	}

	public ReconstructionChildMapping getMapping() {
		return mapping;
	}

	public void setMapping(ReconstructionChildMapping mapping) {
		this.mapping = mapping;
	}

	/**
	 * @return the takeofNode
	 */
	public OutputHostNode getTakeofNode() {
		return this.hostNodes[mapping.getChildTakeof().get(takeofIndex)];
	}

	/**
	 * @return the takeofIndex
	 */
	public int getTakeofIndex() {
		return takeofIndex;
	}

	/**
	 * @param takeofIndex
	 *            the takeofIndex to set
	 */
	public void setTakeofIndex(int takeofIndex) {
		this.takeofIndex = takeofIndex;
	}

	/**
	 * @return the landingNode
	 */
	public OutputHostNode getLandingNode() {
		return this.hostNodes[mapping.getChildLanding().get(landingIndex)];
	}

	/**
	 * @return the landingIndex
	 */
	public int getLandingIndex() {
		return landingIndex;
	}

	/**
	 * @param landingIndex
	 *            the landingIndex to set
	 */
	public void setLandingIndex(int landingIndex) {
		this.landingIndex = landingIndex;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#getToolTipText()
	 */

	public String getToolTipText() {
		return getLabel() + " (" + getRank().toString() + ")";
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.common.api.IPositionNode#getRankText()
	 */

	public String getRankText() {
		return rank.toString();
	}

	public String getHostMapping() {
		String hostMapping = "";
		if (this.hostIndex >= 0)
			hostMapping = hostNodes[this.hostIndex].getLabel();
		return hostMapping;
	}

	/**
	 * @return the yShift
	 */
	public boolean isCospeciation(boolean includeLeaves) {
		return (includeLeaves && this.isLeaf())
				|| (this.hostIndex >= 0 && getDirectEvents().getCospeciations() > 0);
	}

	private int calcYShift(OutputHostNode association) {
		OutputHostNode currentAssociation = getAssociation();
		int result = 0;
		if (!isLeaf()) {
			for (OutputParasiteNode child : getChildren()) {
				result = Math.max(result, child.calcYShift(association));
			}
			if (currentAssociation.equals(association) && !isCospeciation(true))
				result++;
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.wieseke.cptk.output.dao.OutputNode#calcNodePositions(int)
	 */

	public Tupel<Integer, Integer> calcNodePositions(int nextLeafPosX) {
		try {
			IPreferenceStore store = PlatformUI.getPreferenceStore();
			this.xShiftDistance = store
					.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_SHIFT_XDISTANCE);
			this.yShiftDistance = store
					.getInt(OutputPreferenceConstants.OUTPUT_VIEWER_NAME_PARASITE_SHIFT_YDISTANCE);

			OutputHostNode currentAssociation = getAssociation();
			List<Integer> shifts = currentAssociation.getParasiteShifts();

			if (isLeaf()) {
				this.yShift = 0;
				this.xShift = shifts.get(0) + 1;
				// if the parent node is associated with the same host, the
				// current xShift position in each yShift level from the current
				// node's yShift to the parent node's yShift will be blocked
				// because there will be an edge
				if (getParent() != null
						&& getParent().getAssociation().equals(
								currentAssociation)) {
					for (int i = this.yShift; i <= getParent().yShift; i++) {
						if (shifts.size() <= i) {
							shifts.add(this.xShift);
						} else {
							shifts.set(i, this.xShift);
						}
					}
				}
				// the current xShift position at layer 0 (this is a leaf node)
				// will be blocked
				shifts.set(0, this.xShift);
			} else {
				this.yShift = calcYShift(currentAssociation);
				if (getDirectEvents().getCospeciations() != 0) {
					this.yShift = 0;
				}
				this.xShift = Math.max(
						currentAssociation.getMinParasiteShift() + 1, shifts
								.size() > this.yShift ? shifts.get(yShift) + 1
								: 1);
				// the current xShift positions from layer 0 to layer yShift-1
				// will be initialized and set to xShift-1 because there will be
				// at least one edge to descending nodes at this position
				for (int i = 0; i < this.yShift; i++) {
					if (shifts.size() <= i) {
						shifts.add(this.xShift - 1);
					} else {
						shifts.set(i, Math.max(shifts.get(i), this.xShift - 1));
					}
				}
				// the current xShift position at the current layer will be
				// blocked
				if (shifts.size() <= this.yShift) {
					shifts.add(this.xShift);
				} else {
					shifts.set(this.yShift, this.xShift);
				}
				// if the parent node is associated with the same host, the
				// current xShift position in each yShift level from the current
				// node's yShift to the parent node's yShift will be blocked
				// because there will be an edge
				if (getParent() != null
						&& getParent().getAssociation().equals(
								currentAssociation)) {
					for (int i = this.yShift; i <= getParent().yShift; i++) {
						if (shifts.size() <= i) {
							shifts.add(this.xShift);
						} else {
							shifts.set(i, Math.max(shifts.get(i), this.xShift));
						}
					}
				}
				// the children's node and edge positions will be computed
				int index = 0;
				for (OutputParasiteNode child : getChildren()) {
					child.calcNodePositions(-1);
					child.calcEdgePositions(getReconstruction()
							.getChildMappings()[index], getAssociation(),
							isCospeciation(true), child.getTakeofIndex(), child
									.getLandingIndex());
					index++;
				}

			}
			// the exact positions will be set
			setPosX(getAssociation().getPosX() + this.getShiftX());
			setPosY(getAssociation().getPosY() + (getShiftY()));
		} catch (Exception e) {
			CophylogenyLogger
					.logError(
							"Error while computing the positions for OutputParasiteNode",
							e);
		}
		return null;
	}

	/**
	 * for the edge from the current node to the it's parent node the vertices
	 * will be computed and stored into this.edgePositions
	 * 
	 * @param childMapping
	 * @param parentAssociation
	 * @param isCospeciation
	 */
	protected void calcEdgePositions(ReconstructionChildMapping childMapping,
			OutputHostNode parentAssociation, boolean isCospeciation,
			int takeofIndex, int landingIndex) {
		OutputHostNode currentAssociation = getAssociation();
		int xShift = 0;
		int yShift = 0;
		Point edgePosition = null;
		if (currentAssociation != null) {
			this.edgePositions = new ArrayList<Point>();
			OutputHostNode tempAssociation = currentAssociation;
			// if there is no host switch
			if (!childMapping.isHostswitch()) {
				if (!tempAssociation.equals(parentAssociation)) {
					// if there is no host switch occurred and the parent is not
					// associated with the same host, the current xShift
					// position will be blocked for all layers of the host node
					// because there will be an edge at this position at least
					// up to the host's parent
					tempAssociation.setMinParasiteShift(this.xShift);
					tempAssociation = tempAssociation.getParent();
					// for all nodes on the path between the current node's host
					// and the parent node's host there will be an edge and
					// therefore an edge point (at layer 0 of the respective
					// host
					// node), the used xShift position has to be blocked totally
					while (tempAssociation != null
							&& !tempAssociation.equals(parentAssociation)) {
						List<Integer> shifts = tempAssociation
								.getParasiteShifts();
						xShift = Math.max(shifts.get(0) + 1,
								tempAssociation.getMinParasiteShift() + 1);
						edgePosition = new Point(tempAssociation.getPosX()
								+ (xShift * this.xShiftDistance),
								tempAssociation.getPosY());
						shifts.set(0, xShift);
						tempAssociation.setMinParasiteShift(xShift);
						this.edgePositions.add(0, edgePosition);
						tempAssociation = tempAssociation.getParent();
					}
					// if the parent has an yShift other than 0 there has to be
					// also an edgePoint at the parent node's host at layer 0,
					// the used xShift position has to be blocked up to the
					// parents yShift position, because there will be a line
					// from layer 0 to the parent node
					if (!isCospeciation) {
						List<Integer> shifts = tempAssociation
								.getParasiteShifts();
						xShift = Math.max(shifts.get(0) + 1,
								tempAssociation.getMinParasiteShift() + 1);
						edgePosition = new Point(tempAssociation.getPosX()
								+ (xShift * this.xShiftDistance),
								tempAssociation.getPosY());

						for (int i = 0; i < this.getParent().yShift; i++)
							shifts.set(i, xShift);
						// shifts.set(0, xShift);
						this.edgePositions.add(0, edgePosition);
						tempAssociation = tempAssociation.getParent();
					}
				}
			}
			// if there is a host switch
			else {
				OutputHostNode landingAssociation = this.hostNodes[childMapping
						.getChildLanding().get(landingIndex)];
				OutputHostNode takeofAssociation = this.hostNodes[childMapping
						.getChildTakeof().get(takeofIndex)];
				// compute the edge points from the current node's host to the
				// landing host
				if (landingAssociation != null && takeofAssociation != null) {
					if (!tempAssociation.equals(landingAssociation)) {
						tempAssociation = tempAssociation.getParent();
						// all the way from the current node's host up to the
						// landing node there has to be edge points and the
						// used xShift has to be blocked totally
						while (tempAssociation != null
								&& !tempAssociation.equals(landingAssociation)) {
							List<Integer> shifts = tempAssociation
									.getParasiteShifts();
							xShift = Math.max(shifts.get(0) + 1,
									tempAssociation.getMinParasiteShift() + 1);
							edgePosition = new Point(tempAssociation.getPosX()
									+ (xShift * this.xShiftDistance),
									tempAssociation.getPosY());
							shifts.set(0, xShift);
							tempAssociation.setMinParasiteShift(xShift);
							this.edgePositions.add(0, edgePosition);
							tempAssociation = tempAssociation.getParent();
						}
						// for the edge point at the landing host, the
						// used xShift has not to be blocked totally
						List<Integer> shifts = tempAssociation
								.getParasiteShifts();
						xShift = Math.max(shifts.get(0) + 1,
								tempAssociation.getMinParasiteShift() + 1);
						edgePosition = new Point(tempAssociation.getPosX()
								+ (xShift * this.xShiftDistance),
								tempAssociation.getPosY());
						shifts.set(0, xShift);
						this.edgePositions.add(0, edgePosition);
					}
					// as the parasite arrived a little bit earlier (not exactly
					// at the landing host node there has to be an additional
					// edge point at layer 1 (or higher), the used xShift has
					// not
					// to be blocked totally
					List<Integer> shifts = tempAssociation.getParasiteShifts();
					yShift = currentAssociation.equals(landingAssociation) ? this.yShift + 1
							: 1;
					while (shifts.size() <= yShift) {
						shifts.add(0);
					}
					xShift = Math.max(shifts.get(yShift) + 1,
							tempAssociation.getMinParasiteShift() + 1);
					edgePosition = new Point(tempAssociation.getPosX()
							+ getShiftX(xShift, yShift, tempAssociation),
							tempAssociation.getPosY() + getShiftY(yShift));
					shifts.set(yShift, xShift);
					this.edgePositions.add(0, edgePosition);

					// compute the edge points from the take-of host to the
					// parent
					// node's host
					tempAssociation = takeofAssociation;
					// this has to be done only if the host switch was delayed
					if (!parentAssociation.equals(tempAssociation)) {
						// as the host switch does not start at a host node (but
						// a little bit earlier) we assume the layer 1 as
						// take-of
						// point, so there has to be an edge point there
						shifts = tempAssociation.getParasiteShifts();
						yShift = 1;
						while (shifts.size() <= yShift) {
							shifts.add(0);
						}
						xShift = Math.max(shifts.get(yShift) + 1,
								tempAssociation.getMinParasiteShift() + 1);
						edgePosition = new Point(tempAssociation.getPosX()
								+ getShiftX(xShift, yShift, tempAssociation),
								tempAssociation.getPosY() + getShiftY(yShift));
						shifts.set(yShift, xShift);
						this.edgePositions.add(0, edgePosition);
						tempAssociation.setMinParasiteShift(xShift);
						tempAssociation = tempAssociation.getParent();
						// for the rest of the way from the take-of host to the
						// parent node's host there has to be edge points and
						// the
						// used xShift has to be blocked totally
						while (tempAssociation != null
								&& !tempAssociation.equals(parentAssociation)) {
							shifts = tempAssociation.getParasiteShifts();
							int shiftX = shifts.get(0) + 1;
							edgePosition = new Point(tempAssociation.getPosX()
									+ (shiftX * this.xShiftDistance),
									tempAssociation.getPosY());
							shifts.set(0, shiftX);
							tempAssociation.setMinParasiteShift(shiftX);
							this.edgePositions.add(0, edgePosition);
							tempAssociation = tempAssociation.getParent();
						}
						// as the parent node's position is not at layer 0 (as
						// there was a host switch) the has to be an additional
						// edge point at the parent nodes's host at layer 0, the
						// used xShift has not to be blocked totally
						shifts = tempAssociation.getParasiteShifts();
						xShift = Math.max(shifts.get(0) + 1,
								tempAssociation.getMinParasiteShift() + 1);
						edgePosition = new Point(tempAssociation.getPosX()
								+ (xShift * this.xShiftDistance),
								tempAssociation.getPosY());
						shifts.set(0, xShift);
						this.edgePositions.add(0, edgePosition);
					}
				}
			}
		}
	}

	public void initFirstReconstruction(int hostIndex) {
		this.setHostIndex(hostIndex);
		this.reconstructionIndex = 0;
		for (int i = 0; i < getDirectChildrenCount(); i++) {
			OutputParasiteNode child = getChildren().get(i);
			child.initFirstReconstruction(getReconstruction()
					.getChildMappings()[child.getOwnChildIndex()]
					.getChildMapping());
			child.setMapping(getReconstruction().getChildMappings()[child
					.getOwnChildIndex()]);
		}
	}

	public boolean nextReconstruction() {
		for (int i = 0; i < getChildren().size(); i++) {
			OutputParasiteNode child = getChildren().get(i);
			if (child.nextReconstruction())
				return true;
		}
		if (getReconstructions().size() > this.reconstructionIndex + 1) {
			this.reconstructionIndex++;
			for (int i = 0; i < getDirectChildrenCount(); i++) {
				OutputParasiteNode child = getChildren().get(i);
				child.initFirstReconstruction(getReconstruction()
						.getChildMappings()[child.getOwnChildIndex()]
						.getChildMapping());
				child.setMapping(getReconstruction().getChildMappings()[child
						.getOwnChildIndex()]);
			}
			return true;
		}
		return false;
	}

	public void initPreviousReconstruction(int hostIndex) {
		this.setHostIndex(hostIndex);
		this.reconstructionIndex = getReconstructions().size() - 1;
		for (int i = 0; i < getDirectChildrenCount(); i++) {
			OutputParasiteNode child = getChildren().get(i);
			child.initPreviousReconstruction(getReconstruction()
					.getChildMappings()[child.getOwnChildIndex()]
					.getChildMapping());
			child.setMapping(getReconstruction().getChildMappings()[child
					.getOwnChildIndex()]);
		}
	}

	public boolean previousReconstruction() {
		for (int i = getChildren().size() - 1; i >= 0; i--) {
			OutputParasiteNode child = getChildren().get(i);
			if (child.previousReconstruction())
				return true;
		}
		if (reconstructionIndex > 0) {
			reconstructionIndex--;
			for (int i = 0; i < getDirectChildrenCount(); i++) {
				OutputParasiteNode child = getChildren().get(i);
				child.initPreviousReconstruction(getReconstruction()
						.getChildMappings()[child.getOwnChildIndex()]
						.getChildMapping());
				child.setMapping(getReconstruction().getChildMappings()[child
						.getOwnChildIndex()]);

			}
			return true;
		}
		return false;
	}
}
