/**
 * 
 */
package com.wieseke.cptk.output.view;

import org.eclipse.jface.viewers.DoubleClickEvent;
import org.eclipse.jface.viewers.IDoubleClickListener;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.ISelectionListener;
import org.eclipse.ui.IWorkbenchPart;
import org.eclipse.ui.part.ViewPart;

import com.wieseke.cptk.common.constants.OptionsConstants;
import com.wieseke.cptk.common.selection.CophylogenyViewerSelection;
import com.wieseke.cptk.output.dao.OutputCophylogeny;
import com.wieseke.cptk.output.editor.VisualOutputEditor;
import com.wieseke.cptk.output.model.OutputReconstruction;
import com.wieseke.cptk.output.model.OutputReconstructions;
import com.wieseke.cptk.output.provider.ReconstructionContentProvider;
import com.wieseke.cptk.output.provider.ReconstructionLabelProvider;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;

/**
 * @author Nic
 * 
 */
public class ReconstructionsView extends ViewPart {

	public static final String ID = "com.wieseke.cptk.output.view.ReconstructionsView";

	private TreeViewer viewer;

	private ISelectionListener cophylogenySelectionListener;

	/**
	 * 
	 */
	public ReconstructionsView() {

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.ui.part.WorkbenchPart#createPartControl(org.eclipse.swt.widgets
	 * .Composite)
	 */
	
	public void createPartControl(Composite parent) {
		viewer = new TreeViewer(parent, SWT.MULTI | SWT.H_SCROLL | SWT.V_SCROLL
				| SWT.FULL_SELECTION);

		viewer.setContentProvider(new ReconstructionContentProvider(viewer));
		viewer.setLabelProvider(new ReconstructionLabelProvider());
		viewer.addDoubleClickListener(new IDoubleClickListener() {

			
			public void doubleClick(DoubleClickEvent event) {
				if (event.getSelection() instanceof TreeSelection) {
					TreeSelection treeSelection = (TreeSelection) event
							.getSelection();
					if (treeSelection.getFirstElement() instanceof OutputReconstruction) {
						OutputReconstruction outputReconstruction = (OutputReconstruction) treeSelection
								.getFirstElement();
						ReconstructionCophylogeny reconstructionCophylogeny = outputReconstruction
								.getReconstructionCophylogeny();
						reconstructionCophylogeny
								.applyCosts(outputReconstruction.getCosts());
						reconstructionCophylogeny.getOptions().put(
								OptionsConstants.AUTOMATIC_COSTS_ID,
								OptionsConstants.AUTOMATIC_COSTS_0_ID);
						reconstructionCophylogeny.compute(null, null, true);
					}

				}

			}
		});
		// getSite().setSelectionProvider(viewer);
		hookCophylogenySelection();
	}

	private void hookCophylogenySelection() {
		this.cophylogenySelectionListener = new ISelectionListener() {
			/*
			 * (non-Javadoc)
			 * 
			 * @see
			 * org.eclipse.ui.ISelectionListener#selectionChanged(org.eclipse
			 * .ui.IWorkbenchPart, org.eclipse.jface.viewers.ISelection)
			 */
			
			public void selectionChanged(IWorkbenchPart part,
					ISelection selection) {
				cophylogenySelectionChanged(part, selection);
			}
		};
		getSite().getPage().addSelectionListener(
				this.cophylogenySelectionListener);
	}

	protected void cophylogenySelectionChanged(IWorkbenchPart part,
			ISelection selection) {
		if (part == this)
			return;
		if (!(part instanceof VisualOutputEditor)) {
			this.viewer.setInput(null);
			return;
		}
		if (!selection.isEmpty()
				&& selection instanceof CophylogenyViewerSelection) {
			CophylogenyViewerSelection cophylogenySelection = (CophylogenyViewerSelection) selection;
			if (cophylogenySelection.getCophylogenyViewer().getCophylogeny() instanceof OutputCophylogeny) {
				OutputCophylogeny outputCophylogeny = (OutputCophylogeny) cophylogenySelection
						.getCophylogenyViewer().getCophylogeny();
				OutputReconstructions outputReconstructions = new OutputReconstructions(
						outputCophylogeny);
				this.viewer.setInput(outputReconstructions);
				return;
			}
		}

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see org.eclipse.ui.part.WorkbenchPart#setFocus()
	 */
	
	public void setFocus() {
		viewer.getControl().setFocus();
	}

	
	public void dispose() {
		if (this.cophylogenySelectionListener != null) {
			getSite().getPage().removeSelectionListener(
					this.cophylogenySelectionListener);
		}
		super.dispose();
	}

}
