/**
 * 
 */
package com.wieseke.cptk.reconstruction.job;

import java.io.File;
import java.io.OutputStreamWriter;
import java.util.Map;
import java.util.Random;

import org.apache.commons.io.FileUtils;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.core.runtime.jobs.Job;

import com.wieseke.cptk.common.dao.ReconstructionCosts;
import com.wieseke.cptk.common.log.CophylogenyLogger;
import com.wieseke.cptk.input.dao.InputCophylogeny;
import com.wieseke.cptk.reconstruction.dao.ReconstructionCophylogeny;
import com.wieseke.cptk.statistics.TreeGenerator;

/**
 * @author Nicolas Wieseke
 * 
 */
public class RandomBaseSolverJob extends Job {

	private InputCophylogeny cophylogeny;
	private ReconstructionCosts costs;
	private Map<String, String> options;
	private String filename;
	private int randomCycles;
	private int randomMethod;
	private String model;
	private Random random;

	/**
	 * @param name
	 */
	public RandomBaseSolverJob(IFile file, InputCophylogeny cophylogeny,
			ReconstructionCosts costs, Map<String, String> options,
			int randomCycles, int randomMethod, String model, Random random) {
		super(file.getLocation().toString());
		this.filename = file.getLocation().toString();
		this.cophylogeny = cophylogeny;
		this.costs = costs;
		this.options = options;
		this.randomCycles = randomCycles;
		this.randomMethod = randomMethod;
		this.model = model;
		if (random == null) {
			this.random = new Random();
		} else {
			this.random = random;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @seeorg.eclipse.core.runtime.jobs.Job#run(org.eclipse.core.runtime.
	 * IProgressMonitor)
	 */

	protected IStatus run(IProgressMonitor monitor) {
		try {
			monitor.beginTask("Compute reconstructions for "
					+ this.randomCycles + " random data sets",
					randomCycles * 100);

			OutputStreamWriter osw = new OutputStreamWriter(
					FileUtils.openOutputStream(new File(this.filename)));
			osw.write("Counter\tQuality\tTotalCost\tTotalEvents\tCoCost\tCoEvents\tCoProb\tCoFreq\tSoCost\tSoEvents\tSoProb\tSoFreq\tDuCost\tDuEvents\tDuProb\tDuFreq\tHsCost\tHsEvents\tHsProb\tHsFreq\tRootMapping\n");

			for (int i = 0; i < randomCycles && !monitor.isCanceled(); i++) {
				InputCophylogeny cp = new TreeGenerator(this.random)
						.getRandomInputCophylogeny(this.filename + i,
								this.cophylogeny, this.model, this.randomMethod);

				// create .nex file
				// try {
				// OutputStreamWriter ow = new OutputStreamWriter(
				// FileUtils.openOutputStream(new File(filename + i
				// + ".nex")));
				// NexusSerializer ns = new NexusSerializer();
				// ow.write(ns.serialize(cp, false));
				// ow.flush();
				// ow.close();
				// } catch (SerializeException e) {
				// CophylogenyLogger.logError(e);
				// } catch (IOException e) {
				// CophylogenyLogger.logError(e);
				// }

				ReconstructionCophylogeny reconstructionCophylogeny = new ReconstructionCophylogeny(
						cp, this.costs, this.options);
				reconstructionCophylogeny.compute(osw, monitor, false);
				// monitor.worked(1);
				osw.flush();
				// System.gc();
			}
			osw.close();
			monitor.done();
		} catch (Exception e) {
			CophylogenyLogger.logError(e);
		}
		return Status.OK_STATUS;
	}

}
